# -*- coding: utf-8 -*-
'''
    salt.log.handlers
    ~~~~~~~~~~~~~~~~~

    .. versionadded:: 0.17.0

    Custom logging handlers to be used in salt.
'''
from __future__ import absolute_import

# Import python libs
import sys
import logging
import threading
import logging.handlers

# Import salt libs
from salt.log.mixins import NewStyleClassMixIn, ExcInfoOnLogLevelFormatMixIn

log = logging.getLogger(__name__)


if sys.version_info < (2, 7):
    # Since the NullHandler is only available on python >= 2.7, here's a copy
    # with NewStyleClassMixIn so it's also a new style class
    class NullHandler(logging.Handler, NewStyleClassMixIn):
        '''
        This is 1 to 1 copy of python's 2.7 NullHandler
        '''
        def handle(self, record):
            pass

        def emit(self, record):
            pass

        def createLock(self):  # pylint: disable=C0103
            self.lock = None

    logging.NullHandler = NullHandler


class TemporaryLoggingHandler(logging.NullHandler):
    '''
    This logging handler will store all the log records up to its maximum
    queue size at which stage the first messages stored will be dropped.

    Should only be used as a temporary logging handler, while the logging
    system is not fully configured.

    Once configured, pass any logging handlers that should have received the
    initial log messages to the function
    :func:`TemporaryLoggingHandler.sync_with_handlers` and all stored log
    records will be dispatched to the provided handlers.

    .. versionadded:: 0.17.0
    '''

    def __init__(self, level=logging.NOTSET, max_queue_size=10000):
        self.__max_queue_size = max_queue_size
        super(TemporaryLoggingHandler, self).__init__(level=level)
        self.__messages = []

    def handle(self, record):
        self.acquire()
        if len(self.__messages) >= self.__max_queue_size:
            # Loose the initial log records
            self.__messages.pop(0)
        self.__messages.append(record)
        self.release()

    def sync_with_handlers(self, handlers=()):
        '''
        Sync the stored log records to the provided log handlers.
        '''
        if not handlers:
            return

        while self.__messages:
            record = self.__messages.pop(0)
            for handler in handlers:
                if handler.level > record.levelno:
                    # If the handler's level is higher than the log record one,
                    # it should not handle the log record
                    continue
                handler.handle(record)


class StreamHandler(ExcInfoOnLogLevelFormatMixIn, logging.StreamHandler, NewStyleClassMixIn):
    '''
    Stream handler which properly handles exc_info on a per handler basis
    '''


class FileHandler(ExcInfoOnLogLevelFormatMixIn, logging.FileHandler, NewStyleClassMixIn):
    '''
    File handler which properly handles exc_info on a per handler basis
    '''


class SysLogHandler(ExcInfoOnLogLevelFormatMixIn, logging.handlers.SysLogHandler, NewStyleClassMixIn):
    '''
    Syslog handler which properly handles exc_info on a per handler basis
    '''


if sys.version_info > (2, 6):
    class WatchedFileHandler(ExcInfoOnLogLevelFormatMixIn, logging.handlers.WatchedFileHandler, NewStyleClassMixIn):
        '''
        Watched file handler which properly handles exc_info on a per handler basis
        '''


if sys.version_info < (3, 2):
    class QueueHandler(ExcInfoOnLogLevelFormatMixIn, logging.Handler, NewStyleClassMixIn):
        '''
        This handler sends events to a queue. Typically, it would be used together
        with a multiprocessing Queue to centralise logging to file in one process
        (in a multi-process application), so as to avoid file write contention
        between processes.

        This code is new in Python 3.2, but this class can be copy pasted into
        user code for use with earlier Python versions.
        '''

        def __init__(self, queue):
            '''
            Initialise an instance, using the passed queue.
            '''
            logging.Handler.__init__(self)
            self.queue = queue

        def enqueue(self, record):
            '''
            Enqueue a record.

            The base implementation uses put_nowait. You may want to override
            this method if you want to use blocking, timeouts or custom queue
            implementations.
            '''
            self.queue.put_nowait(record)

        def prepare(self, record):
            '''
            Prepares a record for queuing. The object returned by this method is
            enqueued.

            The base implementation formats the record to merge the message
            and arguments, and removes unpickleable items from the record
            in-place.

            You might want to override this method if you want to convert
            the record to a dict or JSON string, or send a modified copy
            of the record while leaving the original intact.
            '''
            # The format operation gets traceback text into record.exc_text
            # (if there's exception data), and also puts the message into
            # record.message. We can then use this to replace the original
            # msg + args, as these might be unpickleable. We also zap the
            # exc_info attribute, as it's no longer needed and, if not None,
            # will typically not be pickleable.
            self.format(record)
            record.msg = record.getMessage()
            record.args = None
            record.exc_info = None
            return record

        def emit(self, record):
            '''
            Emit a record.

            Writes the LogRecord to the queue, preparing it for pickling first.
            '''
            try:
                self.enqueue(self.prepare(record))
            except Exception:
                self.handleError(record)
else:
    class QueueHandler(ExcInfoOnLogLevelFormatMixIn, logging.handlers.QueueHandler):  # pylint: disable=no-member,E0240
        pass



# -*- coding: utf-8 -*-
'''
.. versionadded:: 2016.3.0

Manage macOS local directory passwords and policies.

Note that it is usually better to apply password policies through the creation
of a configuration profile.
'''
# Authentication concepts reference:
# https://developer.apple.com/library/mac/documentation/Networking/Conceptual/Open_Directory/openDirectoryConcepts/openDirectoryConcepts.html#//apple_ref/doc/uid/TP40000917-CH3-CIFCAIBB

from __future__ import absolute_import
from datetime import datetime


try:
    import pwd
    HAS_PWD = True
except ImportError:
    HAS_PWD = False

# Import salt libs
import salt.utils
import logging
import salt.utils.mac_utils
from salt.exceptions import CommandExecutionError

log = logging.getLogger(__name__)  # Start logging

__virtualname__ = 'shadow'


def __virtual__():
    # Is this macOS?
    if not salt.utils.is_darwin():
        return False, 'Not macOS'

    if HAS_PWD:
        return __virtualname__
    else:
        return (False, 'The pwd module failed to load.')


def _get_account_policy(name):
    '''
    Get the entire accountPolicy and return it as a dictionary. For use by this
    module only

    :param str name: The user name

    :return: a dictionary containing all values for the accountPolicy
    :rtype: dict

    :raises: CommandExecutionError on user not found or any other unknown error
    '''

    cmd = 'pwpolicy -u {0} -getpolicy'.format(name)
    try:
        ret = salt.utils.mac_utils.execute_return_result(cmd)
    except CommandExecutionError as exc:
        if 'Error: user <{0}> not found'.format(name) in exc.strerror:
            raise CommandExecutionError('User not found: {0}'.format(name))
        raise CommandExecutionError('Unknown error: {0}'.format(exc.strerror))

    try:
        policy_list = ret.split('\n')[1].split(' ')
        policy_dict = {}
        for policy in policy_list:
            if '=' in policy:
                key, value = policy.split('=')
                policy_dict[key] = value
        return policy_dict
    except IndexError:
        return {}


def _set_account_policy(name, policy):
    '''
    Set a value in the user accountPolicy. For use by this module only

    :param str name: The user name
    :param str policy: The policy to apply

    :return: True if success, otherwise False
    :rtype: bool

    :raises: CommandExecutionError on user not found or any other unknown error
    '''
    cmd = 'pwpolicy -u {0} -setpolicy "{1}"'.format(name, policy)

    try:
        return salt.utils.mac_utils.execute_return_success(cmd)
    except CommandExecutionError as exc:
        if 'Error: user <{0}> not found'.format(name) in exc.strerror:
            raise CommandExecutionError('User not found: {0}'.format(name))
        raise CommandExecutionError('Unknown error: {0}'.format(exc.strerror))


def _get_account_policy_data_value(name, key):
    '''
    Return the value for a key in the accountPolicy section of the user's plist
    file. For use by this module only

    :param str name: The username
    :param str key: The accountPolicy key

    :return: the value contained within the key
    :rtype: str

    :raises: CommandExecutionError on user not found or any other unknown error
    '''
    cmd = 'dscl . -readpl /Users/{0} accountPolicyData {1}'.format(name, key)
    try:
        ret = salt.utils.mac_utils.execute_return_result(cmd)
    except CommandExecutionError as exc:
        if 'eDSUnknownNodeName' in exc.strerror:
            raise CommandExecutionError('User not found: {0}'.format(name))
        raise CommandExecutionError('Unknown error: {0}'.format(exc.strerror))

    return ret


def _convert_to_datetime(unix_timestamp):
    '''
    Converts a unix timestamp to a human readable date/time

    :param float unix_timestamp: A unix timestamp

    :return: A date/time in the format YYYY-mm-dd HH:MM:SS
    :rtype: str
    '''
    try:
        unix_timestamp = float(unix_timestamp)
        return datetime.fromtimestamp(unix_timestamp).strftime('%Y-%m-%d %H:%M:%S')
    except (ValueError, TypeError):
        return 'Invalid Timestamp'


def info(name):
    '''
    Return information for the specified user

    :param str name: the username

    :return: A dictionary containing the user's shadow information
    :rtype: dict

    CLI Example:

    .. code-block:: bash

        salt '*' shadow.info admin
    '''
    try:
        data = pwd.getpwnam(name)
        return {'name': data.pw_name,
                'passwd': data.pw_passwd,
                'account_created': get_account_created(name),
                'login_failed_count': get_login_failed_count(name),
                'login_failed_last': get_login_failed_last(name),
                'lstchg': get_last_change(name),
                'max': get_maxdays(name),
                'expire': get_expire(name),
                'change': get_change(name),
                'min': 'Unavailable',
                'warn': 'Unavailable',
                'inact': 'Unavailable'}

    except KeyError:
        log.debug('User not found: {0}'.format(name))
        return {'name': '',
                'passwd': '',
                'account_created': '',
                'login_failed_count': '',
                'login_failed_last': '',
                'lstchg': '',
                'max': '',
                'expire': '',
                'change': '',
                'min': '',
                'warn': '',
                'inact': ''}


def get_account_created(name):
    '''
    Get the date/time the account was created

    :param str name: the username of the account

    :return: the date/time the account was created (yyyy-mm-dd hh:mm:ss)
    :rtype: str

    :raises: CommandExecutionError on user not found or any other unknown error

    CLI Example:

    .. code-block:: bash

        salt '*' shadow.get_account_created admin
    '''
    ret = _get_account_policy_data_value(name, 'creationTime')

    unix_timestamp = salt.utils.mac_utils.parse_return(ret)

    date_text = _convert_to_datetime(unix_timestamp)

    return date_text


def get_last_change(name):
    '''
    Get the date/time the account was changed

    :param str name: the username of the account

    :return: the date/time the account was modified (yyyy-mm-dd hh:mm:ss)
    :rtype: str

    :raises: CommandExecutionError on user not found or any other unknown error

    CLI Example:

    .. code-block:: bash

        salt '*' shadow.get_last_change admin
    '''
    ret = _get_account_policy_data_value(name, 'passwordLastSetTime')

    unix_timestamp = salt.utils.mac_utils.parse_return(ret)

    date_text = _convert_to_datetime(unix_timestamp)

    return date_text


def get_login_failed_count(name):
    '''
    Get the the number of failed login attempts

    :param str name: the username of the account

    :return: The number of failed login attempts
    :rtype: int

    :raises: CommandExecutionError on user not found or any other unknown error

    CLI Example:

    .. code-block:: bash

        salt '*' shadow.get_login_failed_count admin
    '''
    ret = _get_account_policy_data_value(name, 'failedLoginCount')

    return salt.utils.mac_utils.parse_return(ret)


def get_login_failed_last(name):
    '''
    Get the date/time of the last failed login attempt

    :param str name: the username of the account

    :return: the date/time of the last failed login attempt on this account
    (yyyy-mm-dd hh:mm:ss)
    :rtype: str

    :raises: CommandExecutionError on user not found or any other unknown error

    CLI Example:

    .. code-block:: bash

        salt '*' shadow.get_login_failed_last admin
    '''
    ret = _get_account_policy_data_value(name, 'failedLoginTimestamp')

    unix_timestamp = salt.utils.mac_utils.parse_return(ret)

    date_text = _convert_to_datetime(unix_timestamp)

    return date_text


def set_maxdays(name, days):
    '''
    Set the maximum age of the password in days

    :param str name: the username of the account

    :param int days: the maximum age of the account in days

    :return: True if successful, False if not
    :rtype: bool

    :raises: CommandExecutionError on user not found or any other unknown error

    CLI Example:

    .. code-block:: bash

        salt '*' shadow.set_maxdays admin 90
    '''
    minutes = days * 24 * 60

    _set_account_policy(
        name, 'maxMinutesUntilChangePassword={0}'.format(minutes))

    return get_maxdays(name) == days


def get_maxdays(name):
    '''
    Get the maximum age of the password

    :param str name: the username of the account

    :return: the maximum age of the password in days
    :rtype: int

    :raises: CommandExecutionError on user not found or any other unknown error

    CLI Example:

    .. code-block:: bash

        salt '*' shadow.get_maxdays admin 90
    '''
    policies = _get_account_policy(name)

    if 'maxMinutesUntilChangePassword' in policies:
        max_minutes = policies['maxMinutesUntilChangePassword']
        return int(max_minutes) / 24 / 60

    return 0


def set_mindays(name, days):
    '''
    Set the minimum password age in days. Not available in macOS.

    :param str name: The user name

    :param int days: The number of days

    :return: Will always return False until macOS supports this feature.
    :rtype: bool

    CLI Example:

    .. code-block:: bash

        salt '*' shadow.set_mindays admin 90
    '''
    return False


def set_inactdays(name, days):
    '''
    Set the number if inactive days before the account is locked. Not available
    in macOS

    :param str name: The user name

    :param int days: The number of days

    :return: Will always return False until macOS supports this feature.
    :rtype: bool

    CLI Example:

    .. code-block:: bash

        salt '*' shadow.set_inactdays admin 90
    '''
    return False


def set_warndays(name, days):
    '''
    Set the number of days before the password expires that the user will start
    to see a warning. Not available in macOS

    :param str name: The user name

    :param int days: The number of days

    :return: Will always return False until macOS supports this feature.
    :rtype: bool

    CLI Example:

    .. code-block:: bash

        salt '*' shadow.set_warndays admin 90
    '''
    return False


def set_change(name, date):
    '''
    Sets the date on which the password expires. The user will be required to
    change their password. Format is mm/dd/yyyy

    :param str name: the name of the user account

    :param date date: the date the password will expire. Must be in mm/dd/yyyy
    format.

    :return: True if successful, otherwise False
    :rtype: bool

    :raises: CommandExecutionError on user not found or any other unknown error

    CLI Example:

    .. code-block:: bash

        salt '*' shadow.set_change username 09/21/2016
    '''
    _set_account_policy(
        name, 'usingExpirationDate=1 expirationDateGMT={0}'.format(date))

    return get_change(name) == date


def get_change(name):
    '''
    Gets the date on which the password expires

    :param str name: the name of the user account

    :return: The date the password will expire
    :rtype: str

    :raises: CommandExecutionError on user not found or any other unknown error

    CLI Example:

    .. code-block:: bash

        salt '*' shadow.get_change username
    '''
    policies = _get_account_policy(name)

    if 'expirationDateGMT' in policies:
        return policies['expirationDateGMT']

    return 'Value not set'


def set_expire(name, date):
    '''
    Sets the date on which the account expires. The user will not be able to
    login after this date. Date format is mm/dd/yyyy

    :param str name: the name of the user account

    :param datetime date: the date the account will expire. Format must be
    mm/dd/yyyy

    :return: True if successful, False if not
    :rtype: bool

    :raises: CommandExecutionError on user not found or any other unknown error

    CLI Example:

    .. code-block:: bash

        salt '*' shadow.set_expire username 07/23/2015
    '''
    _set_account_policy(
        name, 'usingHardExpirationDate=1 hardExpireDateGMT={0}'.format(date))

    return get_expire(name) == date


def get_expire(name):
    '''
    Gets the date on which the account expires

    :param str name: the name of the user account

    :return: the date the account expires
    :rtype: str

    :raises: CommandExecutionError on user not found or any other unknown error

    CLI Example:

    .. code-block:: bash

        salt '*' shadow.get_expire username
    '''
    policies = _get_account_policy(name)

    if 'hardExpireDateGMT' in policies:
        return policies['hardExpireDateGMT']

    return 'Value not set'


def del_password(name):
    '''
    Deletes the account password

    :param str name: The user name of the account

    :return: True if successful, otherwise False
    :rtype: bool

    :raises: CommandExecutionError on user not found or any other unknown error

    CLI Example:

    .. code-block:: bash

        salt '*' shadow.del_password username
    '''
    # This removes the password
    cmd = "dscl . -passwd /Users/{0} ''".format(name)
    try:
        salt.utils.mac_utils.execute_return_success(cmd)
    except CommandExecutionError as exc:
        if 'eDSUnknownNodeName' in exc.strerror:
            raise CommandExecutionError('User not found: {0}'.format(name))
        raise CommandExecutionError('Unknown error: {0}'.format(exc.strerror))

    # This is so it looks right in shadow.info
    cmd = "dscl . -create /Users/{0} Password '*'".format(name)
    salt.utils.mac_utils.execute_return_success(cmd)

    return info(name)['passwd'] == '*'


def set_password(name, password):
    '''
    Set the password for a named user (insecure, the password will be in the
    process list while the command is running)

    :param str name: The name of the local user, which is assumed to be in the
    local directory service

    :param str password: The plaintext password to set

    :return: True if successful, otherwise False
    :rtype: bool

    :raises: CommandExecutionError on user not found or any other unknown error

    CLI Example:

    .. code-block:: bash

        salt '*' mac_shadow.set_password macuser macpassword
    '''
    cmd = "dscl . -passwd /Users/{0} '{1}'".format(name, password)
    try:
        salt.utils.mac_utils.execute_return_success(cmd)
    except CommandExecutionError as exc:
        if 'eDSUnknownNodeName' in exc.strerror:
            raise CommandExecutionError('User not found: {0}'.format(name))
        raise CommandExecutionError('Unknown error: {0}'.format(exc.strerror))

    return True



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Rahul Handay <rahulha@saltstack.com>`
'''

# Import Python Libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.helpers import ensure_in_syspath
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

ensure_in_syspath('../../')

# Import Salt Libs
from salt.states import group

# Globals
group.__salt__ = {}
group.__opts__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class GroupTestCase(TestCase):
    '''
        Validate the group state
    '''
    def test_present(self):
        '''
            Test to ensure that a group is present
        '''
        ret = {'name': 'salt',
               'changes': {},
               'result': True,
               'comment': {}
               }

        ret.update({'comment': 'Error: Conflicting options "members" with'
                    ' "addusers" and/or "delusers" can not be used together. ',
                    'result': None})
        self.assertDictEqual(group.present("salt", delusers=True,
                                           members=True), ret)

        ret.update({'comment': 'Error. Same user(s) can not be'
                    ' added and deleted simultaneously'})
        self.assertDictEqual(group.present("salt", addusers=['a'],
                                           delusers=['a']), ret)

        ret.update({'comment': 'The following group attributes are set'
                    ' to be changed:\nkey1: value1\nkey0: value0\n'})
        mock = MagicMock(side_effect=[{'key0': 'value0',
                                       'key1': 'value1'}, False, False, False])
        with patch.object(group, '_changes', mock):
            with patch.dict(group.__opts__, {"test": True}):
                self.assertDictEqual(group.present("salt"), ret)

                ret.update({'comment': 'Group salt set to be added'})
                self.assertDictEqual(group.present("salt"), ret)

            with patch.dict(group.__opts__, {"test": False}):
                mock = MagicMock(return_value=[{'gid': 1, 'name': 'stack'}])
                with patch.dict(group.__salt__, {'group.getent': mock}):
                    ret.update({'result': False,
                                'comment': 'Group salt is not present but'
                                ' gid 1 is already taken by group stack'})
                    self.assertDictEqual(group.present("salt", 1), ret)

                    mock = MagicMock(return_value=False)
                    with patch.dict(group.__salt__, {'group.add': mock}):
                        ret.update({'comment':
                                    'Failed to create new group salt'})
                        self.assertDictEqual(group.present("salt"), ret)

    def test_absent(self):
        '''
            Test to ensure that the named group is absent
        '''
        ret = {'name': 'salt',
               'changes': {},
               'result': True,
               'comment': {}
               }
        mock = MagicMock(side_effect=[True, True, True, False])
        with patch.dict(group.__salt__, {'group.info': mock}):
            with patch.dict(group.__opts__, {"test": True}):
                ret.update({'result': None,
                            'comment': 'Group salt is set for removal'})
                self.assertDictEqual(group.absent("salt"), ret)

            with patch.dict(group.__opts__, {"test": False}):
                mock = MagicMock(side_effect=[True, False])
                with patch.dict(group.__salt__, {'group.delete': mock}):
                    ret.update({'result': True, 'changes': {'salt': ''},
                                'comment': 'Removed group salt'})
                    self.assertDictEqual(group.absent('salt'), ret)

                    ret.update({'changes': {}, 'result': False,
                                'comment': 'Failed to remove group salt'})
                    self.assertDictEqual(group.absent('salt'), ret)

            ret.update({'result': True,
                        'comment': 'Group not present'})
            self.assertDictEqual(group.absent('salt'), ret)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(GroupTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
Support for Stormpath.

.. versionadded:: 2015.8.0
'''

# Import python libs
from __future__ import absolute_import
import pprint

# Import salt libs
import salt.utils


def __virtual__():
    '''
    Only load if the stormpath module is available in __salt__
    '''
    return 'stormpath.create_account' in __salt__


def present(name, **kwargs):
    '''
    Ensure that an account is present and properly configured

    name
        The email address associated with the Stormpath account

    directory_id
        The ID of a directory which the account belongs to. Required.

    password
        Required when creating a new account. If specified, it is advisable to
        reference the password in another database using an ``sdb://`` URL.
        Will NOT update the password if an account already exists.

    givenName
        Required when creating a new account.

    surname
        Required when creating a new account.

    username
        Optional. Must be unique across the owning directory. If not specified,
        the username will default to the email field.

    middleName
        Optional.

    status
        ``enabled`` accounts are able to login to their assigned applications,
        ``disabled`` accounts may not login to applications, ``unverified``
        accounts are disabled and have not verified their email address.

    customData.
        Optional. Must be specified as a dict.
    '''
    # Because __opts__ is not available outside of functions
    backend = __opts__.get('backend', False)
    if not backend and __opts__.get('requests_lib', False):
        salt.utils.warn_until('Oxygen', '"requests_lib:True" has been replaced by "backend:requests", '
                                            'please change your config')
        backend = 'requests'

    if backend == 'requests':
        from requests.exceptions import HTTPError
    elif backend == 'urrlib2':
        from urllib2 import HTTPError
    else:
        from tornado.httpclient import HTTPError

    ret = {'name': name,
           'changes': {},
           'result': None,
           'comment': ''}
    info = {}
    try:
        result = __salt__['stormpath.show_account'](email=name, **kwargs)
        if len(result['items']) > 0:
            info = result['items'][0]
    except HTTPError:
        pass
    needs_update = {}
    if info.get('email', False):
        for field in kwargs.keys():
            if info.get(field, None) != kwargs[field]:
                needs_update[field] = kwargs[field]
        del needs_update['directory_id']
        if 'password' in needs_update:
            del needs_update['password']
        if len(needs_update.keys()) < 1:
            ret['result'] = True
            ret['comment'] = 'Stormpath account {0} already exists and is correct'.format(name)
            return ret
    if __opts__['test']:
        if len(needs_update.keys()) < 1:
            ret['comment'] = 'Stormpath account {0} needs to be created'.format(name)
        else:
            if 'password' in needs_update:
                needs_update['password'] = '**HIDDEN**'
            ret['comment'] = ('Stormpath account {0} needs the following '
                'fields to be updated: '.format(', '.join(needs_update)))
        return ret
    if len(needs_update.keys()) < 1:
        info = __salt__['stormpath.create_account'](email=name, **kwargs)
        comps = info['href'].split('/')
        account_id = comps[-1]
        ret['changes'] = info
        ret['result'] = True
        kwargs['password'] = '**HIDDEN**'
        ret['comment'] = 'Created account ID {0} ({1}): {2}'.format(
            account_id, name, pprint.pformat(kwargs))
        return ret
    comps = info['href'].split('/')
    account_id = comps[-1]
    result = __salt__['stormpath.update_account'](account_id, items=needs_update)
    if result.get('href', None):
        ret['changes'] = needs_update
        ret['result'] = True
        if 'password' in needs_update:
            needs_update['password'] = '**HIDDEN**'
        ret['comment'] = 'Set the following fields for account ID {0} ({1}): {2}'.format(
            account_id, name, pprint.pformat(needs_update))
        return ret
    else:
        ret['result'] = False
        ret['comment'] = 'Failed to set the following fields for account ID {0} ({1}): {2}'.format(
            account_id, name, pprint.pformat(needs_update))
        return ret


def absent(name, directory_id=None):
    '''
    Ensure that an account associated with the given email address is absent.
    Will search all directories for the account, unless a directory_id is
    specified.

    name
        The email address of the account to delete.

    directory_id
        Optional. The ID of the directory that the account is expected to belong
        to. If not specified, then a list of directories will be retrieved, and
        each will be scanned for the account. Specifying a directory_id will
        therefore cut down on the number of requests to Stormpath, and increase
        performance of this state.
    '''
    # Because __opts__ is not available outside of functions
    backend = __opts__.get('backend', False)
    if not backend and __opts__.get('requests_lib', False):
        salt.utils.warn_until('Oxygen', '"requests_lib:True" has been replaced by "backend:requests", '
                                            'please change your config')
        backend = 'requests'

    if backend == 'requests':
        from requests.exceptions import HTTPError
    elif backend == 'urrlib2':
        from urllib2 import HTTPError
    else:
        from tornado.httpclient import HTTPError

    ret = {'name': name,
           'changes': {},
           'result': None,
           'comment': ''}
    info = {}
    if directory_id is None:
        dirs = __salt__['stormpath.list_directories']()
        for dir_ in dirs.get('items', []):
            try:
                comps = dir_.get('href', '').split('/')
                directory_id = comps[-1]
                info = __salt__['stormpath.show_account'](email=name, directory_id=directory_id)
                if len(info.get('items', [])) > 0:
                    info = info['items'][0]
                    break
            except HTTPError:
                pass
    else:
        info = __salt__['stormpath.show_account'](email=name, directory_id=directory_id)
        info = info['items'][0]
    if 'items' in info:
        ret['result'] = True
        ret['comment'] = 'Stormpath account {0} already absent'.format(name)
        return ret
    if __opts__['test']:
        ret['comment'] = 'Stormpath account {0} needs to be deleted'.format(name)
        return ret
    comps = info['href'].split('/')
    account_id = comps[-1]
    if __salt__['stormpath.delete_account'](account_id):
        ret['changes'] = {'deleted': account_id}
        ret['result'] = True
        ret['comment'] = 'Stormpath account {0} was deleted'.format(name)
        return ret
    else:
        ret['result'] = False
        ret['comment'] = 'Failed to delete Stormpath account {0}'.format(name)
        return ret



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
'''
# Import Python libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import skipIf, TestCase
from salttesting.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch
)

from salttesting.helpers import ensure_in_syspath

ensure_in_syspath('../../')

# Import Salt Libs
from salt.states import selinux

selinux.__opts__ = {}
selinux.__salt__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class SelinuxTestCase(TestCase):
    '''
    Test cases for salt.states.selinux
    '''
    # 'mode' function tests: 1

    def test_mode(self):
        '''
        Test to verifies the mode SELinux is running in,
        can be set to enforcing or permissive.
        '''
        ret = {'name': 'unknown',
               'changes': {},
               'result': False,
               'comment': 'unknown is not an accepted mode'}
        self.assertDictEqual(selinux.mode('unknown'), ret)

        mock_en = MagicMock(return_value='Enforcing')
        mock_pr = MagicMock(side_effect=['Permissive', 'Enforcing'])
        with patch.dict(selinux.__salt__,
                        {'selinux.getenforce': mock_en,
                         'selinux.getconfig': mock_en,
                         'selinux.setenforce': mock_pr}):
            comt = ('SELinux is already in Enforcing mode')
            ret = {'name': 'Enforcing', 'comment': comt, 'result': True, 'changes': {}}
            self.assertDictEqual(selinux.mode('Enforcing'), ret)

            with patch.dict(selinux.__opts__, {'test': True}):
                comt = ('SELinux mode is set to be changed to Permissive')
                ret = {'name': 'Permissive', 'comment': comt,
                       'result': None, 'changes': {'new': 'Permissive', 'old': 'Enforcing'}}
                self.assertDictEqual(selinux.mode('Permissive'), ret)

            with patch.dict(selinux.__opts__, {'test': False}):
                comt = ('SELinux has been set to Permissive mode')
                ret = {'name': 'Permissive', 'comment': comt,
                       'result': True, 'changes': {'new': 'Permissive', 'old': 'Enforcing'}}
                self.assertDictEqual(selinux.mode('Permissive'), ret)

                comt = ('Failed to set SELinux to Permissive mode')
                ret.update({'name': 'Permissive', 'comment': comt,
                            'result': False, 'changes': {}})
                self.assertDictEqual(selinux.mode('Permissive'), ret)

    # 'boolean' function tests: 1

    def test_boolean(self):
        '''
        Test to set up an SELinux boolean.
        '''
        name = 'samba_create_home_dirs'
        value = True
        ret = {'name': name,
               'changes': {},
               'result': False,
               'comment': ''}

        mock_en = MagicMock(return_value=[])
        with patch.dict(selinux.__salt__,
                        {'selinux.list_sebool': mock_en}):
            comt = ('Boolean {0} is not available'.format(name))
            ret.update({'comment': comt})
            self.assertDictEqual(selinux.boolean(name, value), ret)

        mock_bools = MagicMock(return_value={name: {'State': 'on',
                                                    'Default': 'on'}})
        with patch.dict(selinux.__salt__,
                        {'selinux.list_sebool': mock_bools}):
            comt = ('None is not a valid value for the boolean')
            ret.update({'comment': comt})
            self.assertDictEqual(selinux.boolean(name, None), ret)

            comt = ('Boolean is in the correct state')
            ret.update({'comment': comt, 'result': True})
            self.assertDictEqual(selinux.boolean(name, value, True), ret)

            comt = ('Boolean is in the correct state')
            ret.update({'comment': comt, 'result': True})
            self.assertDictEqual(selinux.boolean(name, value), ret)

        mock_bools = MagicMock(return_value={name: {'State': 'off',
                                                    'Default': 'on'}})
        mock = MagicMock(side_effect=[True, False])
        with patch.dict(selinux.__salt__,
                        {'selinux.list_sebool': mock_bools,
                         'selinux.setsebool': mock}):
            with patch.dict(selinux.__opts__, {'test': True}):
                comt = ('Boolean samba_create_home_dirs'
                        ' is set to be changed to on')
                ret.update({'comment': comt, 'result': None})
                self.assertDictEqual(selinux.boolean(name, value), ret)

            with patch.dict(selinux.__opts__, {'test': False}):
                comt = ('Boolean samba_create_home_dirs has been set to on')
                ret.update({'comment': comt, 'result': True})
                self.assertDictEqual(selinux.boolean(name, value), ret)

                comt = ('Failed to set the boolean '
                        'samba_create_home_dirs to on')
                ret.update({'comment': comt, 'result': True})
                self.assertDictEqual(selinux.boolean(name, value), ret)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(SelinuxTestCase, needs_daemon=False)



# coding: utf-8

# Import Salt Testing libs
from __future__ import absolute_import
import integration

# Import Salt libs
import salt.wheel


class KeyWheelModuleTest(integration.TestCase, integration.AdaptedConfigurationTestCaseMixIn):
    def setUp(self):
        self.wheel = salt.wheel.Wheel(dict(self.get_config('client_config')))

    def test_list_all(self):
        ret = self.wheel.cmd('key.list_all', print_event=False)
        for host in ['minion', 'sub_minion']:
            self.assertIn(host, ret['minions'])

    def test_gen(self):
        ret = self.wheel.cmd('key.gen', kwarg={'id_': 'soundtechniciansrock'}, print_event=False)

        self.assertIn('pub', ret)
        self.assertIn('priv', ret)
        self.assertTrue(
            ret.get('pub', '').startswith('-----BEGIN PUBLIC KEY-----'))
        self.assertTrue(
            ret.get('priv', '').startswith('-----BEGIN RSA PRIVATE KEY-----'))

if __name__ == '__main__':
    from integration import run_tests
    run_tests(KeyWheelModuleTest, needs_daemon=True)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Rahul Handay <rahulha@saltstack.com>`
'''

# Import Python Libs
from __future__ import absolute_import

# Import Salt Libs
from salt.states import tomcat
from salt.modules import tomcat as tomcatmod

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.helpers import ensure_in_syspath
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

ensure_in_syspath('../../')

# Globals
tomcat.__salt__ = {'tomcat.extract_war_version': tomcatmod.extract_war_version}
tomcat.__opts__ = {}
tomcat.__env__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class TomcatTestCase(TestCase):
    '''
    Validate the tomcat state
    '''

    def test_war_deployed(self):
        '''
        Test to enforce that the WAR will be deployed and
        started in the context path it will make use of WAR versions
        '''
        ret = {'name': 'salt',
               'changes': {},
               'result': True,
               'comment': ''}
        mock_start = MagicMock(return_value='saltstack')
        mock_undeploy = MagicMock(side_effect=['FAIL', 'saltstack'])
        mock_deploy = MagicMock(return_value='deploy')
        mock_ls = MagicMock(side_effect=[{'salt': {'version': 'jenkins-1.20.4',
                                                   'mode': 'running'}},
                                         {'salt': {'version': '1'}},
                                         {'salt': {'version': 'jenkins-1.2.4',
                                                   'mode': 'run'}},
                                         {'salt': {'version': '1'}},
                                         {'salt': {'version': '1'}},
                                         {'salt': {'version': '1'}}])
        with patch.dict(tomcat.__salt__, {"tomcat.ls": mock_ls,
                                          'tomcat.start': mock_start,
                                          'tomcat.undeploy': mock_undeploy,
                                          'tomcat.deploy_war': mock_deploy}):
            ret.update({'comment': 'salt with version 1.20.4'
                                   ' is already deployed'})
            self.assertDictEqual(tomcat.war_deployed('salt',
                                                     'salt://jenkins'
                                                     '-1.20.4.war'), ret)

            with patch.dict(tomcat.__opts__, {"test": True}):
                ret.update({'changes': {'deploy': 'will deploy salt'
                                                  ' with version 1.2.4',
                                        'undeploy': 'undeployed salt'
                                                    ' with version 1'},
                            'result': None, 'comment': ''})
                self.assertDictEqual(tomcat.war_deployed('salt',
                                                         'salt://jenkins'
                                                         '-1.2.4.war'), ret)

            with patch.dict(tomcat.__opts__, {"test": False}):
                ret.update({'changes': {'start': 'starting salt'},
                            'comment': 'saltstack', 'result': False})
                self.assertDictEqual(tomcat.war_deployed('salt',
                                                         'salt://jenkins'
                                                         '-1.2.4.war'), ret)

                ret.update({'changes': {'deploy': 'will deploy salt with'
                                                  ' version 1.2.4',
                                        'undeploy': 'undeployed salt with'
                                                    ' version 1'},
                            'comment': 'FAIL'})
                self.assertDictEqual(tomcat.war_deployed('salt',
                                                         'salt://jenkins'
                                                         '-1.2.4.war'), ret)

                ret.update({'changes': {'undeploy': 'undeployed salt'
                                                    ' with version 1'},
                            'comment': 'deploy'})
                self.assertDictEqual(tomcat.war_deployed('salt',
                                                         'salt://jenkins'
                                                         '-1.2.4.war'), ret)

    def test_war_deployed_no_version(self):
        '''
        Tests that going from versions to no versions and back work, as well
        as not overwriting a WAR without version with another without version.
        '''
        ret = {'name': 'salt',
               'changes': {},
               'result': None,
               'comment': ''}

        mock_deploy = MagicMock(return_value='deploy')
        mock_undeploy = MagicMock(return_value='SUCCESS')
        mock_ls_version = MagicMock(return_value={'salt': {'version': '1.2.4',
                                                           'mode': 'running'}})
        mock_ls_no_version = MagicMock(return_value={'salt': {'version': '',
                                                              'mode': 'running'}})

        # We're just checking what it *would* do.
        with patch.dict(tomcat.__opts__, {"test": True}):
            with patch.dict(tomcat.__salt__,
                            {"tomcat.ls": mock_ls_version,
                             "tomcat.deploy_war": mock_deploy,
                             "tomcat.undeploy": mock_undeploy}):
                # We deploy from version to no version
                ret.update({'changes':
                                {'deploy': 'will deploy salt with no version',
                                 'undeploy': 'undeployed salt with version 1.2.4'},
                            })
                self.assertDictEqual(tomcat.war_deployed('salt',
                                                         'salt://jenkins.war'),
                                     ret)

            with patch.dict(tomcat.__salt__,
                            {"tomcat.ls": mock_ls_no_version,
                             "tomcat.deploy_war": mock_deploy,
                             "tomcat.undeploy": mock_undeploy}):
                # Deploy from none to specified version
                ret.update({'changes':
                                {'deploy': 'will deploy salt with version 1.2.4',
                                 'undeploy': 'undeployed salt with no version'},
                            })
                self.assertDictEqual(tomcat.war_deployed('salt',
                                                         'salt://jenkins.war',
                                                         version='1.2.4'),
                                     ret)
                # Deploy from none to extracted version
                self.assertDictEqual(tomcat.war_deployed('salt',
                                                         'salt://jenkins-1.2.4.war'),
                                     ret)
                # Don't deploy from no version to no version
                ret.update({'changes': {},
                            'comment': 'salt with no version is already deployed',
                            'result': True
                            })
                # Don't deploy from blank to blank version
                self.assertDictEqual(tomcat.war_deployed('salt',
                                                         'salt://jenkins.war'),
                                     ret)
                # Don't deploy from blank to false version
                self.assertDictEqual(tomcat.war_deployed('salt',
                                                         'salt://jenkins-1.2.4.war',
                                                         version=False),
                                     ret)

    def test_wait(self):
        '''
        Test to wait for the tomcat manager to load
        '''
        ret = {'name': 'salt',
               'changes': {},
               'result': True,
               'comment': 'tomcat manager is ready'}
        mock = MagicMock(return_value=True)
        with patch.dict(tomcat.__salt__, {"tomcat.status": mock}):
            self.assertDictEqual(tomcat.wait('salt'), ret)

    def test_mod_watch(self):
        '''
        Test to the tomcat watcher function.
        '''
        ret = {'name': 'salt',
               'changes': {},
               'result': False,
               'comment': 'True'}
        mock = MagicMock(return_value='True')
        with patch.dict(tomcat.__salt__, {"tomcat.reload": mock}):
            ret.update({'changes': {'salt': False}})
            self.assertDictEqual(tomcat.mod_watch('salt'), ret)

    def test_undeployed(self):
        '''
        Test to enforce that the WAR will be un-deployed from the server
        '''
        ret = {'name': 'salt',
               'changes': {},
               'result': False,
               'comment': 'True'}
        mock = MagicMock(side_effect=[False, True, True, True, True])
        mock1 = MagicMock(side_effect=[{'salt': {'a': 1}},
                                       {'salt': {'version': 1}},
                                       {'salt': {'version': 1}},
                                       {'salt': {'version': 1}}])
        mock2 = MagicMock(side_effect=['FAIL', 'saltstack'])
        with patch.dict(tomcat.__salt__, {"tomcat.status": mock,
                                          "tomcat.ls": mock1,
                                          "tomcat.undeploy": mock2}):
            ret.update({'comment': 'Tomcat Manager does not respond'})
            self.assertDictEqual(tomcat.undeployed('salt'), ret)

            ret.update({'comment': '', 'result': True})
            self.assertDictEqual(tomcat.undeployed('salt'), ret)

            with patch.dict(tomcat.__opts__, {"test": True}):
                ret.update({'changes': {'undeploy': 1}, 'result': None})
                self.assertDictEqual(tomcat.undeployed('salt'), ret)

            with patch.dict(tomcat.__opts__, {"test": False}):
                ret.update({'changes': {'undeploy': 1},
                            'comment': 'FAIL', 'result': False})
                self.assertDictEqual(tomcat.undeployed('salt'), ret)

                ret.update({'changes': {'undeploy': 1},
                            'comment': '', 'result': True})
                self.assertDictEqual(tomcat.undeployed('salt'), ret)


if __name__ == '__main__':
    from integration import run_tests

    run_tests(TomcatTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
Tests for the Git state
'''

# Import python libs
from __future__ import absolute_import
import errno
import os
import shutil
import socket
import string
import subprocess
import tempfile

# Import Salt Testing libs
from salttesting.helpers import ensure_in_syspath, skip_if_binaries_missing
ensure_in_syspath('../../')

# Import salt libs
import integration
import salt.utils


@skip_if_binaries_missing('git')
class GitTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
    '''
    Validate the git state
    '''

    def setUp(self):
        super(GitTest, self).setUp()
        self.__domain = 'github.com'
        try:
            if hasattr(socket, 'setdefaulttimeout'):
                # 10 second dns timeout
                socket.setdefaulttimeout(10)
            socket.gethostbyname(self.__domain)
        except socket.error:
            msg = 'error resolving {0}, possible network issue?'
            self.skipTest(msg.format(self.__domain))

    def tearDown(self):
        # Reset the dns timeout after the test is over
        socket.setdefaulttimeout(None)

    def test_latest(self):
        '''
        git.latest
        '''
        name = os.path.join(integration.TMP, 'salt_repo')
        try:
            ret = self.run_state(
                'git.latest',
                name='https://{0}/saltstack/salt-test-repo.git'.format(self.__domain),
                target=name
            )
            self.assertSaltTrueReturn(ret)
            self.assertTrue(os.path.isdir(os.path.join(name, '.git')))
        finally:
            shutil.rmtree(name, ignore_errors=True)

    def test_latest_with_rev_and_submodules(self):
        '''
        git.latest
        '''
        name = os.path.join(integration.TMP, 'salt_repo')
        try:
            ret = self.run_state(
                'git.latest',
                name='https://{0}/saltstack/salt-test-repo.git'.format(self.__domain),
                rev='develop',
                target=name,
                submodules=True
            )
            self.assertSaltTrueReturn(ret)
            self.assertTrue(os.path.isdir(os.path.join(name, '.git')))
        finally:
            shutil.rmtree(name, ignore_errors=True)

    def test_latest_failure(self):
        '''
        git.latest
        '''
        name = os.path.join(integration.TMP, 'salt_repo')
        try:
            ret = self.run_state(
                'git.latest',
                name='https://youSpelledGitHubWrong.com/saltstack/salt-test-repo.git',
                rev='develop',
                target=name,
                submodules=True
            )
            self.assertSaltFalseReturn(ret)
            self.assertFalse(os.path.isdir(os.path.join(name, '.git')))
        finally:
            shutil.rmtree(name, ignore_errors=True)

    def test_latest_empty_dir(self):
        '''
        git.latest
        '''
        name = os.path.join(integration.TMP, 'salt_repo')
        if not os.path.isdir(name):
            os.mkdir(name)
        try:
            ret = self.run_state(
                'git.latest',
                name='https://{0}/saltstack/salt-test-repo.git'.format(self.__domain),
                rev='develop',
                target=name,
                submodules=True
            )
            self.assertSaltTrueReturn(ret)
            self.assertTrue(os.path.isdir(os.path.join(name, '.git')))
        finally:
            shutil.rmtree(name, ignore_errors=True)

    def test_latest_unless_no_cwd_issue_6800(self):
        '''
        cwd=target was being passed to _run_check which blew up if
        target dir did not already exist.
        '''
        name = os.path.join(integration.TMP, 'salt_repo')
        if os.path.isdir(name):
            shutil.rmtree(name)
        try:
            ret = self.run_state(
                'git.latest',
                name='https://{0}/saltstack/salt-test-repo.git'.format(self.__domain),
                rev='develop',
                target=name,
                unless='test -e {0}'.format(name),
                submodules=True
            )
            self.assertSaltTrueReturn(ret)
            self.assertTrue(os.path.isdir(os.path.join(name, '.git')))
        finally:
            shutil.rmtree(name, ignore_errors=True)

    def test_numeric_rev(self):
        '''
        git.latest with numeric revision
        '''
        name = os.path.join(integration.TMP, 'salt_repo')
        try:
            ret = self.run_state(
                'git.latest',
                name='https://{0}/saltstack/salt-test-repo.git'.format(self.__domain),
                rev=0.11,
                target=name,
                submodules=True,
                timeout=120
            )
            self.assertSaltTrueReturn(ret)
            self.assertTrue(os.path.isdir(os.path.join(name, '.git')))
        finally:
            shutil.rmtree(name, ignore_errors=True)

    def test_latest_with_local_changes(self):
        '''
        Ensure that we fail the state when there are local changes and succeed
        when force_reset is True.
        '''
        name = os.path.join(integration.TMP, 'salt_repo')
        try:
            # Clone repo
            ret = self.run_state(
                'git.latest',
                name='https://{0}/saltstack/salt-test-repo.git'.format(self.__domain),
                target=name
            )
            self.assertSaltTrueReturn(ret)
            self.assertTrue(os.path.isdir(os.path.join(name, '.git')))

            # Make change to LICENSE file.
            with salt.utils.fopen(os.path.join(name, 'LICENSE'), 'a') as fp_:
                fp_.write('Lorem ipsum dolor blah blah blah....\n')

            # Make sure that we now have uncommitted changes
            self.assertTrue(self.run_function('git.diff', [name, 'HEAD']))

            # Re-run state with force_reset=False
            ret = self.run_state(
                'git.latest',
                name='https://{0}/saltstack/salt-test-repo.git'.format(self.__domain),
                target=name,
                force_reset=False
            )
            self.assertSaltTrueReturn(ret)
            self.assertEqual(
                ret[next(iter(ret))]['comment'],
                ('Repository {0} is up-to-date, but with local changes. Set '
                 '\'force_reset\' to True to purge local changes.'.format(name))
            )

            # Now run the state with force_reset=True
            ret = self.run_state(
                'git.latest',
                name='https://{0}/saltstack/salt-test-repo.git'.format(self.__domain),
                target=name,
                force_reset=True
            )
            self.assertSaltTrueReturn(ret)

            # Make sure that we no longer have uncommitted changes
            self.assertFalse(self.run_function('git.diff', [name, 'HEAD']))

        finally:
            shutil.rmtree(name, ignore_errors=True)

    def test_latest_fast_forward(self):
        '''
        Test running git.latest state a second time after changes have been
        made to the remote repo.
        '''
        def _head(cwd):
            return self.run_function('git.rev_parse', [cwd, 'HEAD'])

        repo_url = 'https://{0}/saltstack/salt-test-repo.git'.format(self.__domain)
        mirror_dir = os.path.join(integration.TMP, 'salt_repo_mirror')
        mirror_url = 'file://' + mirror_dir
        admin_dir = os.path.join(integration.TMP, 'salt_repo_admin')
        clone_dir = os.path.join(integration.TMP, 'salt_repo')

        try:
            # Mirror the repo
            self.run_function('git.clone',
                              [mirror_dir, repo_url, None, '--mirror'])

            # Make sure the directory for the mirror now exists
            self.assertTrue(os.path.exists(mirror_dir))

            # Clone the mirror twice, once to the admin location and once to
            # the clone_dir
            ret = self.run_state('git.latest', name=mirror_url, target=admin_dir)
            self.assertSaltTrueReturn(ret)
            ret = self.run_state('git.latest', name=mirror_url, target=clone_dir)
            self.assertSaltTrueReturn(ret)

            # Make a change to the repo by editing the file in the admin copy
            # of the repo and committing.
            head_pre = _head(admin_dir)
            with open(os.path.join(admin_dir, 'LICENSE'), 'a') as fp_:
                fp_.write('Hello world!')
            self.run_function('git.commit', [admin_dir, 'Added a line', '-a'])
            # Make sure HEAD is pointing to a new SHA so we know we properly
            # committed our change.
            head_post = _head(admin_dir)
            self.assertNotEqual(head_pre, head_post)

            # Push the change to the mirror
            # NOTE: the test will fail if the salt-test-repo's default branch
            # is changed.
            self.run_function('git.push', [admin_dir, 'origin', 'develop'])

            # Re-run the git.latest state on the clone_dir
            ret = self.run_state('git.latest', name=mirror_url, target=clone_dir)
            self.assertSaltTrueReturn(ret)

            # Make sure that the clone_dir now has the correct SHA
            self.assertEqual(head_post, _head(clone_dir))

        finally:
            for path in (mirror_dir, admin_dir, clone_dir):
                shutil.rmtree(path, ignore_errors=True)

    def _changed_local_branch_helper(self, rev, hint):
        '''
        We're testing two almost identical cases, the only thing that differs
        is the rev used for the git.latest state.
        '''
        name = os.path.join(integration.TMP, 'salt_repo')
        cwd = os.getcwd()
        try:
            # Clone repo
            ret = self.run_state(
                'git.latest',
                name='https://{0}/saltstack/salt-test-repo.git'.format(self.__domain),
                rev=rev,
                target=name
            )
            self.assertSaltTrueReturn(ret)

            # Check out a new branch in the clone and make a commit, to ensure
            # that when we re-run the state, it is not a fast-forward change
            os.chdir(name)
            with salt.utils.fopen(os.devnull, 'w') as devnull:
                subprocess.check_call(['git', 'checkout', '-b', 'new_branch'],
                                      stdout=devnull, stderr=devnull)
                with salt.utils.fopen('foo', 'w'):
                    pass
                subprocess.check_call(['git', 'add', '.'],
                                      stdout=devnull, stderr=devnull)
                subprocess.check_call(['git', 'commit', '-m', 'add file'],
                                      stdout=devnull, stderr=devnull)
            os.chdir(cwd)

            # Re-run the state, this should fail with a specific hint in the
            # comment field.
            ret = self.run_state(
                'git.latest',
                name='https://{0}/saltstack/salt-test-repo.git'.format(self.__domain),
                rev=rev,
                target=name
            )
            self.assertSaltFalseReturn(ret)

            comment = ret[next(iter(ret))]['comment']
            self.assertTrue(hint in comment)
        finally:
            # Make sure that we change back to the original cwd even if there
            # was a traceback in the test.
            os.chdir(cwd)
            shutil.rmtree(name, ignore_errors=True)

    def test_latest_changed_local_branch_rev_head(self):
        '''
        Test for presence of hint in failure message when the local branch has
        been changed and a the rev is set to HEAD

        This test will fail if the default branch for the salt-test-repo is
        ever changed.
        '''
        self._changed_local_branch_helper(
            'HEAD',
            'The default remote branch (develop) differs from the local '
            'branch (new_branch)'
        )

    def test_latest_changed_local_branch_rev_develop(self):
        '''
        Test for presence of hint in failure message when the local branch has
        been changed and a non-HEAD rev is specified
        '''
        self._changed_local_branch_helper(
            'develop',
            'The desired rev (develop) differs from the name of the local '
            'branch (new_branch)'
        )

    def test_latest_updated_remote_rev(self):
        '''
        Ensure that we don't exit early when checking for a fast-forward
        '''
        orig_cwd = os.getcwd()
        name = tempfile.mkdtemp(dir=integration.TMP)
        target = os.path.join(integration.TMP, 'test_latest_updated_remote_rev')

        # Initialize a new git repository
        subprocess.check_call(['git', 'init', '--quiet', name])

        try:
            os.chdir(name)
            # Set user.name and user.email config attributes if not present
            for key, value in (('user.name', 'Jenkins'),
                               ('user.email', 'qa@saltstack.com')):
                # Check if key is missing
                keycheck = subprocess.Popen(
                    ['git', 'config', '--get', key],
                    stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE)
                if keycheck.wait() != 0:
                    # Set the key if it is not present
                    subprocess.check_call(
                        ['git', 'config', key, value])

            # Add and commit a file
            with salt.utils.fopen('foo.txt', 'w') as fp_:
                fp_.write('Hello world\n')
            subprocess.check_call(['git', 'add', '.'])
            subprocess.check_call(['git', 'commit', '-qm', 'init'])

            # Run the state to clone the repo we just created
            ret = self.run_state(
                'git.latest',
                name=name,
                target=target,
            )
            self.assertSaltTrueReturn(ret)

            # Add another commit
            with salt.utils.fopen('foo.txt', 'w') as fp_:
                fp_.write('Added a line\n')
            subprocess.check_call(['git', 'commit', '-aqm', 'added a line'])

            # Run the state again. It should pass, if it doesn't then there was
            # a problem checking whether or not the change is a fast-forward.
            ret = self.run_state(
                'git.latest',
                name=name,
                target=target,
            )
            self.assertSaltTrueReturn(ret)
        finally:
            os.chdir(orig_cwd)
            for path in (name, target):
                try:
                    shutil.rmtree(path)
                except OSError as exc:
                    if exc.errno != errno.ENOENT:
                        raise exc

    def test_present(self):
        '''
        git.present
        '''
        name = os.path.join(integration.TMP, 'salt_repo')
        try:
            ret = self.run_state(
                'git.present',
                name=name,
                bare=True
            )
            self.assertSaltTrueReturn(ret)
            self.assertTrue(os.path.isfile(os.path.join(name, 'HEAD')))
        finally:
            shutil.rmtree(name, ignore_errors=True)

    def test_present_failure(self):
        '''
        git.present
        '''
        name = os.path.join(integration.TMP, 'salt_repo')
        if not os.path.isdir(name):
            os.mkdir(name)
        try:
            fname = os.path.join(name, 'stoptheprocess')

            with salt.utils.fopen(fname, 'a') as fh_:
                fh_.write('')

            ret = self.run_state(
                'git.present',
                name=name,
                bare=True
            )
            self.assertSaltFalseReturn(ret)
            self.assertFalse(os.path.isfile(os.path.join(name, 'HEAD')))
        finally:
            shutil.rmtree(name, ignore_errors=True)

    def test_present_empty_dir(self):
        '''
        git.present
        '''
        name = os.path.join(integration.TMP, 'salt_repo')
        if not os.path.isdir(name):
            os.mkdir(name)
        try:
            ret = self.run_state(
                'git.present',
                name=name,
                bare=True
            )
            self.assertSaltTrueReturn(ret)
            self.assertTrue(os.path.isfile(os.path.join(name, 'HEAD')))
        finally:
            shutil.rmtree(name, ignore_errors=True)

    def test_config_set_value_with_space_character(self):
        '''
        git.config
        '''
        name = tempfile.mkdtemp(dir=integration.TMP)
        self.addCleanup(shutil.rmtree, name, ignore_errors=True)
        subprocess.check_call(['git', 'init', '--quiet', name])

        ret = self.run_state(
            'git.config_set',
            name='user.name',
            value='foo bar',
            repo=name,
            **{'global': False})
        self.assertSaltTrueReturn(ret)


@skip_if_binaries_missing('git')
class LocalRepoGitTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
    '''
    Tests which do no require connectivity to github.com
    '''
    def test_renamed_default_branch(self):
        '''
        Test the case where the remote branch has been removed
        https://github.com/saltstack/salt/issues/36242
        '''
        cwd = os.getcwd()
        repo = tempfile.mkdtemp(dir=integration.TMP)
        admin = tempfile.mkdtemp(dir=integration.TMP)
        name = tempfile.mkdtemp(dir=integration.TMP)
        for dirname in (repo, admin, name):
            self.addCleanup(shutil.rmtree, dirname, ignore_errors=True)
        self.addCleanup(os.chdir, cwd)

        with salt.utils.fopen(os.devnull, 'w') as devnull:
            # Create bare repo
            subprocess.check_call(['git', 'init', '--bare', repo],
                                  stdout=devnull, stderr=devnull)
            # Clone bare repo
            subprocess.check_call(['git', 'clone', repo, admin],
                                  stdout=devnull, stderr=devnull)

            # Create, add, commit, and push file
            os.chdir(admin)
            with salt.utils.fopen('foo', 'w'):
                pass
            subprocess.check_call(['git', 'add', '.'],
                                  stdout=devnull, stderr=devnull)
            subprocess.check_call(['git', 'commit', '-m', 'init'],
                                  stdout=devnull, stderr=devnull)
            subprocess.check_call(['git', 'push', 'origin', 'master'],
                                  stdout=devnull, stderr=devnull)

        # Change back to the original cwd
        os.chdir(cwd)

        # Rename remote 'master' branch to 'develop'
        os.rename(
            os.path.join(repo, 'refs', 'heads', 'master'),
            os.path.join(repo, 'refs', 'heads', 'develop')
        )

        # Run git.latest state. This should successfuly clone and fail with a
        # specific error in the comment field.
        ret = self.run_state(
            'git.latest',
            name=repo,
            target=name,
            rev='develop',
        )
        self.assertSaltFalseReturn(ret)
        self.assertEqual(
            ret[next(iter(ret))]['comment'],
            'Remote HEAD refers to a ref that does not exist. '
            'This can happen when the default branch on the '
            'remote repository is renamed or deleted. If you '
            'are unable to fix the remote repository, you can '
            'work around this by setting the \'branch\' argument '
            '(which will ensure that the named branch is created '
            'if it does not already exist).\n\n'
            'Changes already made: {0} cloned to {1}'
            .format(repo, name)
        )
        self.assertEqual(
            ret[next(iter(ret))]['changes'],
            {'new': '{0} => {1}'.format(repo, name)}
        )

        # Run git.latest state again. This should fail again, with a different
        # error in the comment field, and should not change anything.
        ret = self.run_state(
            'git.latest',
            name=repo,
            target=name,
            rev='develop',
        )
        self.assertSaltFalseReturn(ret)
        self.assertEqual(
            ret[next(iter(ret))]['comment'],
            'Cannot set/unset upstream tracking branch, local '
            'HEAD refers to nonexistent branch. This may have '
            'been caused by cloning a remote repository for which '
            'the default branch was renamed or deleted. If you '
            'are unable to fix the remote repository, you can '
            'work around this by setting the \'branch\' argument '
            '(which will ensure that the named branch is created '
            'if it does not already exist).'
        )
        self.assertEqual(ret[next(iter(ret))]['changes'], {})

        # Run git.latest state again with a branch manually set. This should
        # checkout a new branch and the state should pass.
        ret = self.run_state(
            'git.latest',
            name=repo,
            target=name,
            rev='develop',
            branch='develop',
        )
        # State should succeed
        self.assertSaltTrueReturn(ret)
        self.assertSaltCommentRegexpMatches(
            ret,
            'New branch \'develop\' was checked out, with origin/develop '
            r'\([0-9a-f]{7}\) as a starting point'
        )
        # Only the revision should be in the changes dict.
        self.assertEqual(
            list(ret[next(iter(ret))]['changes'].keys()),
            ['revision']
        )
        # Since the remote repo was incorrectly set up, the local head should
        # not exist (therefore the old revision should be None).
        self.assertEqual(
            ret[next(iter(ret))]['changes']['revision']['old'],
            None
        )
        # Make sure the new revision is a SHA (40 chars, all hex)
        self.assertTrue(
            len(ret[next(iter(ret))]['changes']['revision']['new']) == 40)
        self.assertTrue(
            all([x in string.hexdigits for x in
                 ret[next(iter(ret))]['changes']['revision']['new']])
        )


if __name__ == '__main__':
    from integration import run_tests
    run_tests(GitTest)



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import

# Import Salt Libs
from salt.modules import mac_keychain as keychain

# Import Salt Testing Libs
from salttesting import TestCase
from salttesting.helpers import ensure_in_syspath
from salttesting.mock import (
    MagicMock,
    patch
)

ensure_in_syspath('../../')

keychain.__salt__ = {}


class KeychainTestCase(TestCase):

    def test_install_cert(self):
        '''
            Test installing a certificate into the macOS keychain
        '''
        mock = MagicMock()
        with patch.dict(keychain.__salt__, {'cmd.run': mock}):
            keychain.install('/path/to/cert.p12', 'passw0rd')
            mock.assert_called_once_with('security import /path/to/cert.p12 -P passw0rd '
                                         '-k /Library/Keychains/System.keychain')

    @patch('salt.modules.mac_keychain.unlock_keychain')
    def test_install_cert_extras(self, unlock_mock):
        '''
            Test installing a certificate into the macOS keychain with extras
        '''
        mock = MagicMock()
        with patch.dict(keychain.__salt__, {'cmd.run': mock}):
            keychain.install('/path/to/cert.p12', 'passw0rd', '/path/to/chain', allow_any=True,
                             keychain_password='passw0rd1')
            unlock_mock.assert_called_once_with('/path/to/chain', 'passw0rd1')
            mock.assert_called_once_with('security import /path/to/cert.p12 -P passw0rd -k /path/to/chain -A')

    def test_uninstall_cert(self):
        '''
            Test uninstalling a certificate from the macOS keychain
        '''
        mock = MagicMock()
        with patch.dict(keychain.__salt__, {'cmd.run': mock}):
            keychain.uninstall('/path/to/cert.p12', 'passw0rd')
            mock.assert_called_once_with('security delete-certificate -c "/path/to/cert.p12" passw0rd')

    def test_list_certs(self):
        '''
            Test listing available certificates in a keychain
        '''
        expected = ["com.apple.systemdefault", "com.apple.kerberos.kdc"]
        mock = MagicMock(return_value='"com.apple.systemdefault"\n"com.apple.kerberos.kdc"')
        with patch.dict(keychain.__salt__, {'cmd.run': mock}):
            out = keychain.list_certs('/path/to/cert.p12')
            mock.assert_called_once_with('security find-certificate -a /path/to/cert.p12 | '
                                         'grep -o "alis".*\\" | grep -o \'\\"[-A-Za-z0-9.:() ]*\\"\'',
                                         python_shell=True)

            self.assertEqual(out, expected)

    def test_get_friendly_name(self):
        '''
            Test getting the friendly name of a certificate
        '''
        expected = "ID Installer Salt"
        mock = MagicMock(return_value="friendlyName: ID Installer Salt")
        with patch.dict(keychain.__salt__, {'cmd.run': mock}):
            out = keychain.get_friendly_name('/path/to/cert.p12', 'passw0rd')
            mock.assert_called_once_with('openssl pkcs12 -in /path/to/cert.p12 -passin pass:passw0rd -info '
                                         '-nodes -nokeys 2> /dev/null | grep friendlyName:',
                                         python_shell=True)

            self.assertEqual(out, expected)

    def test_get_default_keychain(self):
        '''
            Test getting the default keychain
        '''
        mock = MagicMock()
        with patch.dict(keychain.__salt__, {'cmd.run': mock}):
            keychain.get_default_keychain('frank', 'system')
            mock.assert_called_once_with('security default-keychain -d system', runas='frank')

    def test_set_default_keychain(self):
        '''
            Test setting the default keychain
        '''
        mock = MagicMock()
        with patch.dict(keychain.__salt__, {'cmd.run': mock}):
            keychain.set_default_keychain('/path/to/chain.keychain', 'system', 'frank')
            mock.assert_called_once_with('security default-keychain -d system -s /path/to/chain.keychain',
                                         runas='frank')

    def test_unlock_keychain(self):
        '''
            Test unlocking the keychain
        '''
        mock = MagicMock()
        with patch.dict(keychain.__salt__, {'cmd.run': mock}):
            keychain.unlock_keychain('/path/to/chain.keychain', 'passw0rd')
            mock.assert_called_once_with('security unlock-keychain -p passw0rd /path/to/chain.keychain')

if __name__ == '__main__':
    from integration import run_tests
    run_tests(KeychainTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
Library for interacting with Slack API

.. versionadded:: 2016.3.0

:configuration: This module can be used by specifying the name of a
    configuration profile in the minion config, minion pillar, or master
    config.

    For example:

    .. code-block:: yaml

        slack:
          api_key: peWcBiMOS9HrZG15peWcBiMOS9HrZG15
'''
from __future__ import absolute_import

import logging
# Import 3rd-party libs
# pylint: disable=import-error,no-name-in-module,redefined-builtin
from salt.ext.six.moves.urllib.parse import urljoin as _urljoin
from salt.ext.six.moves.urllib.parse import urlencode as _urlencode
import salt.ext.six.moves.http_client
from salt.version import __version__
# pylint: enable=import-error,no-name-in-module

log = logging.getLogger(__name__)


def query(function,
          token=None,
          api_version='1',
          method='POST',
          header_dict=None,
          data=None,
          query_params=None,
          opts=None):
    '''
    PushOver object method function to construct and execute on the API URL.

    :param token:       The PushOver api key.
    :param api_version: The PushOver API version to use, defaults to version 1.
    :param function:    The PushOver api function to perform.
    :param method:      The HTTP method, e.g. GET or POST.
    :param data:        The data to be sent for POST method.
    :return:            The json response from the API call or False.
    '''

    ret = {'message': '',
           'res': True}

    pushover_functions = {
        'message': {
            'request': 'messages.json',
            'response': 'status',
        },
        'validate_user': {
            'request': 'users/validate.json',
            'response': 'status',
        },
        'validate_sound': {
            'request': 'sounds.json',
            'response': 'status',
        },
    }

    api_url = 'https://api.pushover.net'
    base_url = _urljoin(api_url, api_version + '/')
    path = pushover_functions.get(function).get('request')
    url = _urljoin(base_url, path, False)

    if not query_params:
        query_params = {}

    decode = True
    if method == 'DELETE':
        decode = False

    result = salt.utils.http.query(
        url,
        method,
        params=query_params,
        data=data,
        header_dict=header_dict,
        decode=decode,
        decode_type='json',
        text=True,
        status=True,
        cookies=True,
        persist_session=True,
        opts=opts,
    )

    if result.get('status', None) == salt.ext.six.moves.http_client.OK:
        response = pushover_functions.get(function).get('response')
        if response in result and result[response] == 0:
            ret['res'] = False
        ret['message'] = result
        return ret
    else:
        try:
            if 'response' in result and result[response] == 0:
                ret['res'] = False
            ret['message'] = result
        except ValueError:
            ret['res'] = False
            ret['message'] = result
        return ret


def validate_sound(sound,
                   token):
    '''
    Send a message to a Pushover user or group.
    :param sound:       The sound that we want to verify
    :param token:       The PushOver token.
    '''
    ret = {
            'message': 'Sound is invalid',
            'res': False
           }
    parameters = dict()
    parameters['token'] = token

    response = query(function='validate_sound',
                     method='GET',
                     query_params=parameters)

    if response['res']:
        if 'message' in response:
            _message = response.get('message', '')
            if 'status' in _message:
                if _message.get('dict', {}).get('status', '') == 1:
                    sounds = _message.get('dict', {}).get('sounds', '')
                    if sound in sounds:
                        ret['message'] = 'Valid sound {0}.'.format(sound)
                        ret['res'] = True
                    else:
                        ret['message'] = 'Warning: {0} not a valid sound.'.format(sound)
                        ret['res'] = False
                else:
                    ret['message'] = ''.join(_message.get('dict', {}).get('errors'))
    return ret


def validate_user(user,
                  device,
                  token):
    '''
    Send a message to a Pushover user or group.
    :param user:        The user or group name, either will work.
    :param device:      The device for the user.
    :param token:       The PushOver token.
    '''
    res = {
            'message': 'User key is invalid',
            'result': False
           }

    parameters = dict()
    parameters['user'] = user
    parameters['token'] = token
    if device:
        parameters['device'] = device

    response = query(function='validate_user',
                     method='POST',
                     header_dict={'Content-Type': 'application/x-www-form-urlencoded'},
                     data=_urlencode(parameters))

    if response['res']:
        if 'message' in response:
            _message = response.get('message', '')
            if 'status' in _message:
                if _message.get('dict', {}).get('status', None) == 1:
                    res['result'] = True
                    res['message'] = 'User key is valid.'
                else:
                    res['result'] = False
                    res['message'] = ''.join(_message.get('dict', {}).get('errors'))
    return res



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import
from distutils.version import LooseVersion  # pylint: disable=import-error,no-name-in-module
import random
import string

# Import Salt Testing libs
from salttesting.unit import skipIf, TestCase
from salttesting.mock import (
    MagicMock,
    NO_MOCK,
    NO_MOCK_REASON,
    patch
)
from salttesting.helpers import ensure_in_syspath

ensure_in_syspath('../../')

# Import Salt libs
import salt.config
import salt.loader
from salt.ext.six.moves import range  # pylint: disable=import-error,redefined-builtin

# Import 3rd-party libs
import logging

# pylint: disable=import-error,no-name-in-module,unused-import
from unit.modules.boto_cloudtrail_test import BotoCloudTrailTestCaseMixin

# Import 3rd-party libs
try:
    import boto
    import boto3
    from botocore.exceptions import ClientError
    HAS_BOTO = True
except ImportError:
    HAS_BOTO = False

# pylint: enable=import-error,no-name-in-module,unused-import

# the boto_cloudtrail module relies on the connect_to_region() method
# which was added in boto 2.8.0
# https://github.com/boto/boto/commit/33ac26b416fbb48a60602542b4ce15dcc7029f12
required_boto3_version = '1.2.1'

log = logging.getLogger(__name__)

opts = salt.config.DEFAULT_MINION_OPTS
context = {}
utils = salt.loader.utils(opts, whitelist=['boto3'], context=context)
serializers = salt.loader.serializers(opts)
funcs = salt.loader.minion_mods(opts, context=context, utils=utils, whitelist=['boto_cloudtrail'])
salt_states = salt.loader.states(opts=opts, functions=funcs, utils=utils, whitelist=['boto_cloudtrail'], serializers=serializers)


def _has_required_boto():
    '''
    Returns True/False boolean depending on if Boto is installed and correct
    version.
    '''
    if not HAS_BOTO:
        return False
    elif LooseVersion(boto3.__version__) < LooseVersion(required_boto3_version):
        return False
    else:
        return True

if _has_required_boto():
    region = 'us-east-1'
    access_key = 'GKTADJGHEIQSXMKKRBJ08H'
    secret_key = 'askdjghsdfjkghWupUjasdflkdfklgjsdfjajkghs'
    conn_parameters = {'region': region, 'key': access_key, 'keyid': secret_key, 'profile': {}}
    error_message = 'An error occurred (101) when calling the {0} operation: Test-defined error'
    not_found_error = ClientError({
        'Error': {
            'Code': 'TrailNotFoundException',
            'Message': "Test-defined error"
        }
    }, 'msg')
    error_content = {
      'Error': {
        'Code': 101,
        'Message': "Test-defined error"
      }
    }
    trail_ret = dict(Name='testtrail',
                     IncludeGlobalServiceEvents=True,
                     KmsKeyId=None,
                     LogFileValidationEnabled=False,
                     S3BucketName='auditinfo',
                     TrailARN='arn:aws:cloudtrail:us-east-1:214351231622:trail/testtrail')
    status_ret = dict(IsLogging=False,
                      LatestCloudWatchLogsDeliveryError=None,
                      LatestCloudWatchLogsDeliveryTime=None,
                      LatestDeliveryError=None,
                      LatestDeliveryTime=None,
                      LatestDigestDeliveryError=None,
                      LatestDigestDeliveryTime=None,
                      LatestNotificationError=None,
                      LatestNotificationTime=None,
                      StartLoggingTime=None,
                      StopLoggingTime=None)


@skipIf(HAS_BOTO is False, 'The boto module must be installed.')
@skipIf(_has_required_boto() is False, 'The boto3 module must be greater than'
                                       ' or equal to version {0}'
        .format(required_boto3_version))
@skipIf(NO_MOCK, NO_MOCK_REASON)
class BotoCloudTrailStateTestCaseBase(TestCase):
    conn = None

    # Set up MagicMock to replace the boto3 session
    def setUp(self):
        context.clear()
        # connections keep getting cached from prior tests, can't find the
        # correct context object to clear it. So randomize the cache key, to prevent any
        # cache hits
        conn_parameters['key'] = ''.join(random.choice(string.ascii_lowercase + string.digits) for _ in range(50))

        self.patcher = patch('boto3.session.Session')
        self.addCleanup(self.patcher.stop)
        mock_session = self.patcher.start()

        session_instance = mock_session.return_value
        self.conn = MagicMock()
        session_instance.client.return_value = self.conn


class BotoCloudTrailTestCase(BotoCloudTrailStateTestCaseBase, BotoCloudTrailTestCaseMixin):
    '''
    TestCase for salt.modules.boto_cloudtrail state.module
    '''

    def test_present_when_trail_does_not_exist(self):
        '''
        Tests present on a trail that does not exist.
        '''
        self.conn.get_trail_status.side_effect = [not_found_error, status_ret]
        self.conn.create_trail.return_value = trail_ret
        self.conn.describe_trails.return_value = {'trailList': [trail_ret]}
        with patch.dict(funcs, {'boto_iam.get_account_id': MagicMock(return_value='1234')}):
            result = salt_states['boto_cloudtrail.present'](
                         'trail present',
                         Name=trail_ret['Name'],
                         S3BucketName=trail_ret['S3BucketName'])

        self.assertTrue(result['result'])
        self.assertEqual(result['changes']['new']['trail']['Name'],
                         trail_ret['Name'])

    def test_present_when_trail_exists(self):
        self.conn.get_trail_status.return_value = status_ret
        self.conn.create_trail.return_value = trail_ret
        self.conn.describe_trails.return_value = {'trailList': [trail_ret]}
        with patch.dict(funcs, {'boto_iam.get_account_id': MagicMock(return_value='1234')}):
            result = salt_states['boto_cloudtrail.present'](
                         'trail present',
                         Name=trail_ret['Name'],
                         S3BucketName=trail_ret['S3BucketName'],
                         LoggingEnabled=False)
        self.assertTrue(result['result'])
        self.assertEqual(result['changes'], {})

    def test_present_with_failure(self):
        self.conn.get_trail_status.side_effect = [not_found_error, status_ret]
        self.conn.create_trail.side_effect = ClientError(error_content, 'create_trail')
        with patch.dict(funcs, {'boto_iam.get_account_id': MagicMock(return_value='1234')}):
            result = salt_states['boto_cloudtrail.present'](
                         'trail present',
                         Name=trail_ret['Name'],
                         S3BucketName=trail_ret['S3BucketName'],
                         LoggingEnabled=False)
        self.assertFalse(result['result'])
        self.assertTrue('An error occurred' in result['comment'])

    def test_absent_when_trail_does_not_exist(self):
        '''
        Tests absent on a trail that does not exist.
        '''
        self.conn.get_trail_status.side_effect = not_found_error
        result = salt_states['boto_cloudtrail.absent']('test', 'mytrail')
        self.assertTrue(result['result'])
        self.assertEqual(result['changes'], {})

    def test_absent_when_trail_exists(self):
        self.conn.get_trail_status.return_value = status_ret
        result = salt_states['boto_cloudtrail.absent']('test', trail_ret['Name'])
        self.assertTrue(result['result'])
        self.assertEqual(result['changes']['new']['trail'], None)

    def test_absent_with_failure(self):
        self.conn.get_trail_status.return_value = status_ret
        self.conn.delete_trail.side_effect = ClientError(error_content, 'delete_trail')
        result = salt_states['boto_cloudtrail.absent']('test', trail_ret['Name'])
        self.assertFalse(result['result'])
        self.assertTrue('An error occurred' in result['comment'])



# -*- coding: utf-8 -*-
'''
Management zpool

:maintainer:    Jorge Schrauwen <sjorge@blackdot.be>
:maturity:      new
:depends:       zpool
:platform:      smartos, illumos, solaris, freebsd, linux

.. versionadded:: 2016.3.0

.. code-block:: yaml

    oldpool:
      zpool.absent:
        - export: true

    newpool:
      zpool.present:
        - config:
            import: false
            force: true
        - properties:
            comment: salty storage pool
        - layout:
            mirror-0:
              /dev/disk0
              /dev/disk1
            mirror-1:
              /dev/disk2
              /dev/disk3

    simplepool:
      zpool.present:
        - config:
            import: false
            force: true
        - properties:
            comment: another salty storage pool
        - layout:
            - /dev/disk0
            - /dev/disk1

.. warning::

    The layout will never be updated, it will only be used at time of creation.
    It's a whole lot of work to figure out if a devices needs to be detached, removed, ... this is best done by the sysadmin on a case per case basis.

    Filesystem properties are also not updated, this should be managed by the zfs state module.

'''
from __future__ import absolute_import

# Import Python libs
import os
import logging

# Import Salt libs
from salt.utils.odict import OrderedDict

log = logging.getLogger(__name__)

# Define the state's virtual name
__virtualname__ = 'zpool'


def __virtual__():
    '''
    Provides zpool state
    '''
    if 'zpool.create' in __salt__:
        return True
    else:
        return (
            False,
            '{0} state module can only be loaded on illumos, Solaris, SmartOS, FreeBSD, ...'.format(
                __virtualname__
            )
        )


def present(name, properties=None, filesystem_properties=None, layout=None, config=None):
    '''
    ensure storage pool is present on the system

    name : string
        name of storage pool
    properties : dict
        optional set of properties to set for the storage pool
    filesystem_properties : dict
        optional set of filesystem properties to set for the storage pool (creation only)
    layout: dict
        disk layout to use if the pool does not exist (creation only)
    config : dict
        fine grain control over this state

    .. note::

        The following configuration properties can be toggled in the config parameter.
          - import (true) - try to import the pool before creating it if absent
          - import_dirs (None) - specify additional locations to scan for devices on import
          - device_dir (None, SunOS=/dev/rdsk) - specify device directory to use if not absolute path
          - force (false) - try to force the import or creation

    .. note::

        Because ID's inside the layout dict must be unique they need to have a suffix.

        .. code-block:: yaml

            mirror-0:
              /tmp/vdisk3
              /tmp/vdisk2
            mirror-1:
              /tmp/vdisk0
              /tmp/vdisk1

        The above yaml will always result in the following zpool create:

        .. code-block:: bash

            zpool create mypool mirror /tmp/vdisk3 /tmp/vdisk2 mirror /tmp/vdisk0 /tmp/vdisk1

    .. warning::

        Pay attention to the order of your dict!

        .. code-block:: yaml

            mirror-0:
              /tmp/vdisk0
              /tmp/vdisk1
            /tmp/vdisk2:

        The above will result in the following zpool create:

        .. code-block:: bash

            zpool create mypool mirror /tmp/vdisk0 /tmp/vdisk1 /tmp/vdisk2

        Creating a 3-way mirror! Why you probably expect it to be mirror root vdev with 2 devices + a root vdev of 1 device!

    '''
    name = name.lower()
    ret = {'name': name,
           'changes': {},
           'result': None,
           'comment': ''}

    # config defaults
    state_config = config if config else {}
    config = {
        'import': True,
        'import_dirs': None,
        'device_dir': None,
        'force': False
    }
    if __grains__['kernel'] == 'SunOS':
        config['device_dir'] = '/dev/rdsk'
    elif __grains__['kernel'] == 'Linux':
        config['device_dir'] = '/dev'
    config.update(state_config)
    log.debug('zpool.present::{0}::config - {1}'.format(name, config))

    # parse layout
    if layout:
        for root_dev in layout:
            if '-' not in root_dev:
                continue
            layout[root_dev] = layout[root_dev].keys() if isinstance(layout[root_dev], OrderedDict) else layout[root_dev].split(' ')

        log.debug('zpool.present::{0}::layout - {1}'.format(name, layout))

    # ensure the pool is present
    ret['result'] = False
    if __salt__['zpool.exists'](name):  # update
        ret['result'] = True

        # retrieve current properties
        properties_current = __salt__['zpool.get'](name)[name]

        # figure out if updates needed
        properties_update = []
        for prop in properties:
            if prop not in properties_current:
                continue

            value = properties[prop]
            if isinstance(value, bool):
                value = 'on' if value else 'off'

            if properties_current[prop] != value:
                properties_update.append(prop)

        # update properties
        for prop in properties_update:
            value = properties[prop]
            res = __salt__['zpool.set'](name, prop, value)

            # also transform value so we match with the return
            if isinstance(value, bool):
                value = 'on' if value else 'off'
            elif ' ' in value:
                value = "'{0}'".format(value)

            # check return
            if name in res and prop in res[name] and res[name][prop] == value:
                if name not in ret['changes']:
                    ret['changes'][name] = {}
                ret['changes'][name].update(res[name])
            else:
                ret['result'] = False
                if ret['comment'] == '':
                    ret['comment'] = 'The following properties were not updated:'
                ret['comment'] = '{0} {1}'.format(ret['comment'], prop)

        if ret['result']:
            ret['comment'] = 'properties updated' if len(ret['changes']) > 0 else 'no update needed'

    else:  # import or create
        if config['import']:  # try import
            log.debug('zpool.present::{0}::importing'.format(name))
            ret['result'] = __salt__['zpool.import'](
                name,
                force=config['force'],
                dir=config['import_dirs']
            )
            ret['result'] = ret['result'].get(name) == 'imported'
            if ret['result']:
                ret['changes'][name] = 'imported'
                ret['comment'] = 'storage pool {0} was imported'.format(name)

        if not ret['result']:  # create
            if not layout:
                ret['comment'] = 'storage pool {0} was not imported, no layout specified for creation'.format(name)
            else:
                log.debug('zpool.present::{0}::creating'.format(name))
                if __opts__['test']:
                    ret['result'] = True
                else:
                    # construct *vdev parameter for zpool.create
                    params = []
                    params.append(name)
                    for root_dev in layout:
                        if '-' in root_dev:  # special device
                            # NOTE: accomidate non existing 'disk' vdev
                            if root_dev.split('-')[0] != 'disk':
                                params.append(root_dev.split('-')[0])  # add the type by stripping the ID
                            for sub_dev in layout[root_dev]:  # add all sub devices
                                if '/' not in sub_dev and config['device_dir'] and os.path.exists(config['device_dir']):
                                    sub_dev = os.path.join(config['device_dir'], sub_dev)
                                params.append(sub_dev)
                        else:  # normal device
                            if '/' not in root_dev and config['device_dir'] and os.path.exists(config['device_dir']):
                                root_dev = os.path.join(config['device_dir'], root_dev)
                            params.append(root_dev)

                    # execute zpool.create
                    ret['result'] = __salt__['zpool.create'](*params, force=config['force'], properties=properties, filesystem_properties=filesystem_properties)
                    if ret['result'].get(name).startswith('created'):
                        ret['result'] = True
                    else:
                        if ret['result'].get(name):
                            ret['comment'] = ret['result'].get(name)
                        ret['result'] = False

                if ret['result']:
                    ret['changes'][name] = 'created'
                    ret['comment'] = 'storage pool {0} was created'.format(name)

    return ret


def absent(name, export=False, force=False):
    '''
    ensure storage pool is absent on the system

    name : string
        name of storage pool
    export : boolean
        export instread of destroy the zpool if present
    force : boolean
        force destroy or export

    '''
    name = name.lower()
    ret = {'name': name,
           'changes': {},
           'result': None,
           'comment': ''}

    # config defaults
    log.debug('zpool.absent::{0}::config::force = {1}'.format(name, force))
    log.debug('zpool.absent::{0}::config::export = {1}'.format(name, export))

    # ensure the pool is absent
    if __salt__['zpool.exists'](name):  # looks like we need to do some work
        ret['result'] = False

        if export:  # try to export the zpool
            if __opts__['test']:
                ret['result'] = True
            else:
                ret['result'] = __salt__['zpool.export'](name, force=force)
                ret['result'] = ret['result'].get(name) == 'exported'

        else:  # try to destroy the zpool
            if __opts__['test']:
                ret['result'] = True
            else:
                ret['result'] = __salt__['zpool.destroy'](name, force=force)
                ret['result'] = ret['result'].get(name) == 'destroyed'

        if ret['result']:  # update the changes and comment
            ret['changes'][name] = 'exported' if export else 'destroyed'
            ret['comment'] = 'storage pool {0} was {1}'.format(name, ret['changes'][name])

    else:  # we are looking good
        ret['result'] = True
        ret['comment'] = 'storage pool {0} is absent'.format(name)

    return ret

# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4



# -*- coding: utf-8 -*-
'''
This roster resolves hostname in a pdsh/clustershell style.

:depends: clustershell, https://github.com/cea-hpc/clustershell

When you want to use host globs for target matching, use ``--roster clustershell``. For example:

.. code-block:: bash

    salt-ssh --roster clustershell 'server_[1-10,21-30],test_server[5,7,9]' test.ping

'''

# Import python libs
from __future__ import absolute_import
import socket
from salt.ext.six.moves import map  # pylint: disable=import-error,redefined-builtin

REQ_ERROR = None
try:
    from ClusterShell.NodeSet import NodeSet
except (ImportError, OSError) as e:
    REQ_ERROR = 'ClusterShell import error, perhaps missing python ClusterShell package'


def __virtual__():
    return (REQ_ERROR is None, REQ_ERROR)


def targets(tgt, tgt_type='glob', **kwargs):
    '''
    Return the targets
    '''
    ret = {}
    ports = __opts__['ssh_scan_ports']
    if not isinstance(ports, list):
        # Comma-separate list of integers
        ports = list(map(int, str(ports).split(',')))

    hosts = list(NodeSet(tgt))
    host_addrs = dict([(h, socket.gethostbyname(h)) for h in hosts])

    for host, addr in host_addrs.items():
        addr = str(addr)
        for port in ports:
            try:
                sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                sock.settimeout(float(__opts__['ssh_scan_timeout']))
                sock.connect((addr, port))
                sock.shutdown(socket.SHUT_RDWR)
                sock.close()
                ret[host] = {'host': host, 'port': port}
            except socket.error:
                pass
    return ret



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Rahul Handay <rahulha@saltstack.com>`
'''

# Import Python Libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.helpers import ensure_in_syspath
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

ensure_in_syspath('../../')

# Import Salt Libs
from salt.states import service

# Globals
service.__salt__ = {}
service.__opts__ = {}


def func(name):
    '''
        Mock func method
    '''
    return name


@skipIf(NO_MOCK, NO_MOCK_REASON)
class ServiceTestCase(TestCase):
    '''
        Validate the service state
    '''
    def test_running(self):
        '''
            Test to verify that the service is running
        '''
        ret = [{'comment': '', 'changes': {}, 'name': 'salt', 'result': True},
               {'changes': {},
                'comment': 'The service salt is already running',
                'name': 'salt', 'result': True},
               {'changes': 'saltstack',
                'comment': 'The service salt is already running',
                'name': 'salt', 'result': True},
               {'changes': {},
                'comment': 'Service salt is set to start', 'name': 'salt',
                'result': None},
               {'changes': 'saltstack',
                'comment': 'Started Service salt', 'name': 'salt',
                'result': True},
               {'changes': {},
                'comment': 'The service salt is already running',
                'name': 'salt', 'result': True},
               {'changes': 'saltstack',
                'comment': 'Service salt failed to start', 'name': 'salt',
                'result': True}]

        tmock = MagicMock(return_value=True)
        fmock = MagicMock(return_value=False)
        vmock = MagicMock(return_value="salt")
        with patch.object(service, '_enabled_used_error', vmock):
            self.assertEqual(service.running("salt", enabled=1), 'salt')

        with patch.object(service, '_available', fmock):
            self.assertDictEqual(service.running("salt"), ret[0])

        with patch.object(service, '_available', tmock):
            with patch.dict(service.__opts__, {'test': False}):
                with patch.dict(service.__salt__, {'service.enabled': tmock,
                                                   'service.status': tmock}):
                    self.assertDictEqual(service.running("salt"), ret[1])

                mock = MagicMock(return_value={'changes': 'saltstack'})
                with patch.dict(service.__salt__, {'service.enabled': MagicMock(side_effect=[False, True]),
                                                   'service.status': tmock}):
                    with patch.object(service, '_enable', mock):
                        self.assertDictEqual(service.running("salt", True), ret[2])

                with patch.dict(service.__salt__, {'service.enabled': MagicMock(side_effect=[True, False]),
                                                   'service.status': tmock}):
                    with patch.object(service, '_disable', mock):
                        self.assertDictEqual(service.running("salt", False), ret[2])

                with patch.dict(service.__salt__, {'service.status': MagicMock(side_effect=[False, True]),
                                                   'service.enabled': MagicMock(side_effect=[False, True]),
                                                   'service.start': MagicMock(return_value="stack")}):
                    with patch.object(service, '_enable', MagicMock(return_value={'changes': 'saltstack'})):
                        self.assertDictEqual(service.running("salt", True), ret[4])

            with patch.dict(service.__opts__, {'test': True}):
                with patch.dict(service.__salt__, {'service.status': tmock}):
                    self.assertDictEqual(service.running("salt"), ret[5])

                with patch.dict(service.__salt__, {'service.status': fmock}):
                    self.assertDictEqual(service.running("salt"), ret[3])

            with patch.dict(service.__opts__, {'test': False}):
                with patch.dict(service.__salt__, {'service.status': MagicMock(side_effect=[False, False]),
                                                   'service.enabled': MagicMock(side_effecct=[True, True]),
                                                   'service.start': MagicMock(return_value='stack')}):
                    with patch.object(service, '_enable', MagicMock(return_value={'changes': 'saltstack'})):
                        self.assertDictEqual(service.running('salt', True), ret[6])

    def test_dead(self):
        '''
            Test to ensure that the named service is dead
        '''
        ret = [{'changes': {}, 'comment': '', 'name': 'salt', 'result': True},
               {'changes': 'saltstack',
                'comment': 'The service salt is already dead', 'name': 'salt',
                'result': True},
               {'changes': {},
                'comment': 'Service salt is set to be killed', 'name': 'salt',
                'result': None},
               {'changes': 'saltstack',
                'comment': 'Service salt was killed', 'name': 'salt',
                'result': True},
               {'changes': {},
                'comment': 'Service salt failed to die', 'name': 'salt',
                'result': False},
               {'changes': 'saltstack',
                'comment': 'The service salt is already dead', 'name': 'salt',
                'result': True}]

        mock = MagicMock(return_value="salt")
        with patch.object(service, '_enabled_used_error', mock):
            self.assertEqual(service.dead("salt", enabled=1), 'salt')

        tmock = MagicMock(return_value=True)
        fmock = MagicMock(return_value=False)
        with patch.object(service, '_available', fmock):
            self.assertDictEqual(service.dead("salt"), ret[0])

        with patch.object(service, '_available', tmock):
            mock = MagicMock(return_value={'changes': 'saltstack'})
            with patch.dict(service.__opts__, {'test': True}):
                with patch.dict(service.__salt__, {'service.enabled': fmock,
                                                   'service.stop': tmock,
                                                   'service.status': fmock}):
                    with patch.object(service, '_enable', mock):
                        self.assertDictEqual(service.dead("salt", True), ret[5])

                with patch.dict(service.__salt__, {'service.enabled': tmock,
                                                   'service.status': tmock}):
                    self.assertDictEqual(service.dead("salt"), ret[2])

            with patch.dict(service.__opts__, {'test': False}):
                with patch.dict(service.__salt__, {'service.enabled': fmock,
                                                   'service.stop': tmock,
                                                   'service.status': fmock}):
                    with patch.object(service, '_enable', mock):
                        self.assertDictEqual(service.dead("salt", True), ret[1])

                with patch.dict(service.__salt__, {'service.enabled': MagicMock(side_effect=[True, True, False]),
                                                   'service.status': MagicMock(side_effect=[True, False, False]),
                                                   'service.stop': MagicMock(return_value="stack")}):
                    with patch.object(service, '_enable', MagicMock(return_value={'changes': 'saltstack'})):
                        self.assertDictEqual(service.dead("salt", True), ret[3])

                # test an initd which a wrong status (True even if dead)
                with patch.dict(service.__salt__, {'service.enabled': MagicMock(side_effect=[False, False, False]),
                                                   'service.status': MagicMock(side_effect=[True, True, True]),
                                                   'service.stop': MagicMock(return_value="stack")}):
                    with patch.object(service, '_disable', MagicMock(return_value={})):
                        self.assertDictEqual(service.dead("salt", False), ret[4])

    def test_dead_with_missing_service(self):
        '''
        Tests the case in which a service.dead state is executed on a state
        which does not exist.

        See https://github.com/saltstack/salt/issues/37511
        '''
        name = 'thisisnotarealservice'
        with patch.dict(service.__salt__,
                        {'service.available': MagicMock(return_value=False)}):
            ret = service.dead(name=name)
            self.assertDictEqual(
                ret,
                {'changes': {},
                 'comment': 'The named service {0} is not available'.format(name),
                 'result': True,
                 'name': name}
            )

    def test_enabled(self):
        '''
            Test to verify that the service is enabled
        '''
        ret = {'changes': 'saltstack', 'comment': '', 'name': 'salt',
               'result': True}
        mock = MagicMock(return_value={'changes': 'saltstack'})
        with patch.object(service, '_enable', mock):
            self.assertDictEqual(service.enabled("salt"), ret)

    def test_disabled(self):
        '''
            Test to verify that the service is disabled
        '''
        ret = {'changes': 'saltstack', 'comment': '', 'name': 'salt',
               'result': True}
        mock = MagicMock(return_value={'changes': 'saltstack'})
        with patch.object(service, '_disable', mock):
            self.assertDictEqual(service.disabled("salt"), ret)

    def test_mod_watch(self):
        '''
            Test to the service watcher, called to invoke the watch command.
        '''
        ret = [{'changes': {},
                'comment': 'Service is already stopped', 'name': 'salt',
                'result': True},
               {'changes': {},
                'comment': 'Unable to trigger watch for service.stack',
                'name': 'salt', 'result': False},
               {'changes': {},
                'comment': 'Service is set to be started', 'name': 'salt',
                'result': None},
               {'changes': {'salt': 'salt'},
                'comment': 'Service started', 'name': 'salt',
                'result': 'salt'}]

        mock = MagicMock(return_value=False)
        with patch.dict(service.__salt__, {'service.status': mock}):
            self.assertDictEqual(service.mod_watch("salt", "dead"), ret[0])

            with patch.dict(service.__salt__, {'service.start': func}):
                with patch.dict(service.__opts__, {'test': True}):
                    self.assertDictEqual(service.mod_watch("salt", "running"),
                                         ret[2])

                with patch.dict(service.__opts__, {'test': False}):
                    self.assertDictEqual(service.mod_watch("salt", "running"),
                                         ret[3])

        self.assertDictEqual(service.mod_watch("salt", "stack"), ret[1])


if __name__ == '__main__':
    from integration import run_tests
    run_tests(ServiceTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import

# Import Salt Testing libs
from salttesting import skipIf, TestCase
from salttesting.helpers import ensure_in_syspath
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

ensure_in_syspath('../../')

# Import Salt libs
import salt.ext.six as six
from salt.utils.odict import OrderedDict
from salt.modules import pillar as pillarmod


pillar_value_1 = dict(a=1, b='very secret')


class PillarModuleTestCase(TestCase):

    def test_obfuscate_inner_recursion(self):
        self.assertEqual(
                pillarmod._obfuscate_inner(dict(a=[1, 2],
                                                b=dict(pwd='secret', deeper=('a', 1)))),
                dict(a=['<int>', '<int>'],
                     b=dict(pwd='<str>', deeper=('<str>', '<int>')))
        )

    def test_obfuscate_inner_more_types(self):
        self.assertEqual(pillarmod._obfuscate_inner(OrderedDict([('key', 'value')])),
                         OrderedDict([('key', '<str>')]))

        self.assertEqual(pillarmod._obfuscate_inner(set((1, 2))),
                         set(['<int>']))

        self.assertEqual(pillarmod._obfuscate_inner((1, 2)),
                         ('<int>', '<int>'))

    @skipIf(NO_MOCK, NO_MOCK_REASON)
    @patch('salt.modules.pillar.items', MagicMock(return_value=pillar_value_1))
    def test_obfuscate(self):
        self.assertEqual(pillarmod.obfuscate(),
                         dict(a='<int>', b='<str>'))

    @skipIf(NO_MOCK, NO_MOCK_REASON)
    @patch('salt.modules.pillar.items', MagicMock(return_value=pillar_value_1))
    def test_ls(self):
        if six.PY3:
            self.assertCountEqual(pillarmod.ls(), ['a', 'b'])
        else:
            self.assertEqual(pillarmod.ls(), ['a', 'b'])

    @skipIf(NO_MOCK, NO_MOCK_REASON)
    def test_pillar_get_default_merge(self):
        defaults = {'int': 1,
                    'string': 'foo',
                    'list': ['foo'],
                    'dict': {'foo': 'bar', 'subkey': {'foo': 'bar'}}}

        pillarmod.__opts__ = {}
        pillarmod.__pillar__ = {'int': 2,
                                'string': 'bar',
                                'list': ['bar', 'baz'],
                                'dict': {'baz': 'qux', 'subkey': {'baz': 'qux'}}}

        # Test that we raise a KeyError when pillar_raise_on_missing is True
        with patch.dict(pillarmod.__opts__, {'pillar_raise_on_missing': True}):
            self.assertRaises(KeyError, pillarmod.get, 'missing')
        # Test that we return an empty string when it is not
        self.assertEqual(pillarmod.get('missing'), '')

        # Test with no default passed (it should be KeyError) and merge=True.
        # The merge should be skipped and the value returned from __pillar__
        # should be returned.
        for item in pillarmod.__pillar__:
            self.assertEqual(
                pillarmod.get(item, merge=True),
                pillarmod.__pillar__[item]
            )

        # Test merging when the type of the default value is not the same as
        # what was returned. Merging should be skipped and the value returned
        # from __pillar__ should be returned.
        for default_type in defaults:
            for data_type in ('dict', 'list'):
                if default_type == data_type:
                    continue
                self.assertEqual(
                    pillarmod.get(item, default=defaults[default_type], merge=True),
                    pillarmod.__pillar__[item]
                )

        # Test recursive dict merging
        self.assertEqual(
            pillarmod.get('dict', default=defaults['dict'], merge=True),
            {'foo': 'bar', 'baz': 'qux', 'subkey': {'foo': 'bar', 'baz': 'qux'}}
        )

        # Test list merging
        self.assertEqual(
            pillarmod.get('list', default=defaults['list'], merge=True),
            ['foo', 'bar', 'baz']
        )

    def test_pillar_get_default_merge_regression_38558(self):
        """Test for pillar.get(key=..., default=..., merge=True)

        Do not update the ``default`` value when using ``merge=True``.

        See: https://github.com/saltstack/salt/issues/38558
        """

        pillarmod.__opts__ = {}
        pillarmod.__pillar__ = {'l1': {'l2': {'l3': 42}}}

        res = pillarmod.get(key='l1')
        self.assertEqual({'l2': {'l3': 42}}, res)

        default = {'l2': {'l3': 43}}

        res = pillarmod.get(key='l1', default=default)
        self.assertEqual({'l2': {'l3': 42}}, res)
        self.assertEqual({'l2': {'l3': 43}}, default)

        res = pillarmod.get(key='l1', default=default, merge=True)
        self.assertEqual({'l2': {'l3': 42}}, res)
        self.assertEqual({'l2': {'l3': 43}}, default)

    def test_pillar_get_default_merge_regression_39062(self):
        '''
        Confirm that we do not raise an exception if default is None and
        merge=True.

        See https://github.com/saltstack/salt/issues/39062 for more info.
        '''
        pillarmod.__opts__ = {}
        pillarmod.__pillar__ = {'foo': 'bar'}

        self.assertEqual(
            pillarmod.get(key='foo', default=None, merge=True),
            'bar',
        )


# gracinet: not sure this is really useful, but other test modules have this as well
if __name__ == '__main__':
    from integration import run_tests
    run_tests(PillarModuleTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import
import os
import string
import random

# Import Salt Testing libs
from salttesting import skipIf
from salttesting.helpers import (
    destructiveTest,
    ensure_in_syspath,
    requires_system_grains
)
ensure_in_syspath('../../')

# Import salt libs
import salt.utils
import integration

# Import 3rd-party libs
from salt.ext.six.moves import range  # pylint: disable=import-error,redefined-builtin

IS_ADMIN = False
if salt.utils.is_windows():
    import salt.utils.win_functions
    current_user = salt.utils.win_functions.get_current_user()
    if current_user == 'SYSTEM':
        IS_ADMIN = True
    else:
        IS_ADMIN = salt.utils.win_functions.is_admin(current_user)
else:
    IS_ADMIN = os.geteuid() == 0


@destructiveTest
@skipIf(not salt.utils.is_linux(), 'These tests can only be run on linux')
@skipIf(not IS_ADMIN, 'You must be root to run these tests')
class UseraddModuleTestLinux(integration.ModuleCase):

    def setUp(self):
        super(UseraddModuleTestLinux, self).setUp()
        os_grain = self.run_function('grains.item', ['kernel'])
        if os_grain['kernel'] not in ('Linux', 'Darwin'):
            self.skipTest(
                'Test not applicable to \'{kernel}\' kernel'.format(
                    **os_grain
                )
            )

    def __random_string(self, size=6):
        return 'RS-' + ''.join(
            random.choice(string.ascii_uppercase + string.digits)
            for x in range(size)
        )

    @requires_system_grains
    def test_groups_includes_primary(self, grains=None):
        # Let's create a user, which usually creates the group matching the
        # name
        uname = self.__random_string()
        if self.run_function('user.add', [uname]) is not True:
            # Skip because creating is not what we're testing here
            self.run_function('user.delete', [uname, True, True])
            self.skipTest('Failed to create user')

        try:
            uinfo = self.run_function('user.info', [uname])
            if grains['os_family'] in ('Suse',):
                self.assertIn('users', uinfo['groups'])
            else:
                self.assertIn(uname, uinfo['groups'])

            # This uid is available, store it
            uid = uinfo['uid']

            self.run_function('user.delete', [uname, True, True])

            # Now, a weird group id
            gname = self.__random_string()
            if self.run_function('group.add', [gname]) is not True:
                self.run_function('group.delete', [gname, True, True])
                self.skipTest('Failed to create group')

            ginfo = self.run_function('group.info', [gname])

            # And create the user with that gid
            if self.run_function('user.add', [uname, uid, ginfo['gid']]) is False:
                # Skip because creating is not what we're testing here
                self.run_function('user.delete', [uname, True, True])
                self.skipTest('Failed to create user')

            uinfo = self.run_function('user.info', [uname])
            self.assertIn(gname, uinfo['groups'])

        except AssertionError:
            self.run_function('user.delete', [uname, True, True])
            raise

    def test_user_primary_group(self, grains=None):
        '''
        Tests the primary_group function
        '''
        name = 'saltyuser'

        # Create a user to test primary group function
        if self.run_function('user.add', [name]) is not True:
            self.run_function('user.delete', [name])
            self.skipTest('Failed to create a user')

        try:
            # Test useradd.primary_group
            primary_group = self.run_function('user.primary_group', [name])
            uid_info = self.run_function('user.info', [name])
            self.assertIn(primary_group, uid_info['groups'])

        except:
            self.run_function('user.delete', [name])
            raise


@destructiveTest
@skipIf(not salt.utils.is_windows(), 'These tests can only be run on Windows')
@skipIf(not IS_ADMIN, 'You must be Administrator to run these tests')
class UseraddModuleTestWindows(integration.ModuleCase):

    def __random_string(self, size=6):
        return 'RS-' + ''.join(
            random.choice(string.ascii_uppercase + string.digits)
            for x in range(size))

    def test_add_user(self):
        '''
        Test adding a user
        '''
        user_name = self.__random_string()

        try:
            if self.run_function('user.add', [user_name]) is False:
                self.run_function('user.delete', [user_name, True, True])
                self.skipTest('Failed to create user')

            user_list = self.run_function('user.list_users')
            self.assertIn(user_name, user_list)

        except AssertionError:
            raise

        finally:
            self.run_function('user.delete', [user_name, True, True])

    def test_add_group(self):
        '''
        Test adding a user
        '''
        group_name = self.__random_string()
        try:
            if self.run_function('group.add', [group_name]) is False:
                # Skip because creating is not what we're testing here
                self.run_function('group.delete', [group_name, True, True])
                self.skipTest('Failed to create group')

            group_list = self.run_function('group.list_groups')
            self.assertIn(group_name, group_list)

        except AssertionError:
            raise

        finally:
            self.run_function('group.delete', [group_name])

    def test_add_user_to_group(self):
        '''
        Test adding a user to a group
        '''
        group_name = self.__random_string()
        user_name = self.__random_string()
        try:
            # Let's create a group
            if self.run_function(
                    'group.add', [group_name])['result'] is not True:
                self.run_function('group.delete', [group_name, True, True])
                self.skipTest('Failed to create group')

            # And create the user as a member of that group
            if self.run_function(
                    'user.add',
                    [user_name, 'groups={0}'.format(group_name)]) is False:
                self.run_function('user.delete', [user_name, True, True])
                self.skipTest('Failed to create user')

            user_info = self.run_function('user.info', [user_name])
            self.assertIn(group_name, user_info['groups'])

        except AssertionError:
            raise

        finally:
            self.run_function('user.delete', [user_name, True, True])
            self.run_function('group.delete', [group_name])


if __name__ == '__main__':
    from integration import run_tests
    run_tests(UseraddModuleTestLinux,
              UseraddModuleTestWindows)



# coding: utf-8

# Python libs
from __future__ import absolute_import

# Salt libs
from salt.beacons import adb

# Salt testing libs
from salttesting import skipIf, TestCase
from salttesting.helpers import ensure_in_syspath
from salttesting.mock import NO_MOCK, NO_MOCK_REASON, patch, Mock

# Globals

adb.__salt__ = {}

ensure_in_syspath('../../')


@skipIf(NO_MOCK, NO_MOCK_REASON)
class ADBBeaconTestCase(TestCase):
    '''
    Test case for salt.beacons.adb
    '''
    def setUp(self):
        adb.last_state = {}
        adb.last_state_extra = {'no_devices': False}

    def test_no_adb_command(self):
        with patch('salt.utils.which') as mock:
            mock.return_value = None

            ret = adb.__virtual__()

            mock.assert_called_once_with('adb')
            self.assertFalse(ret)

    def test_with_adb_command(self):
        with patch('salt.utils.which') as mock:
            mock.return_value = '/usr/bin/adb'

            ret = adb.__virtual__()

            mock.assert_called_once_with('adb')
            self.assertEqual(ret, 'adb')

    def test_non_dict_config(self):
        config = []

        log_mock = Mock()
        adb.log = log_mock

        ret = adb.beacon(config)

        self.assertEqual(ret, [])
        log_mock.info.assert_called_once_with('Configuration for adb beacon must be a dict.')

    def test_empty_config(self):
        config = {}

        log_mock = Mock()
        adb.log = log_mock

        ret = adb.beacon(config)

        self.assertEqual(ret, [])
        log_mock.info.assert_called_once_with('Configuration for adb beacon must include a states array.')

    def test_invalid_states(self):
        config = {'states': ['Random', 'Failings']}

        log_mock = Mock()
        adb.log = log_mock

        ret = adb.beacon(config)

        self.assertEqual(ret, [])
        log_mock.info.assert_called_once_with('Need a one of the following adb states:'
                                              ' offline, bootloader, device, host, recovery, '
                                              'no permissions, sideload, unauthorized, unknown, missing')

    def test_device_state(self):
        config = {'states': ['device']}

        mock = Mock(return_value='List of devices attached\nHTC\tdevice',)
        with patch.dict(adb.__salt__, {'cmd.run': mock}):
            ret = adb.beacon(config)
            self.assertEqual(ret, [{'device': 'HTC', 'state': 'device', 'tag': 'device'}])

    def test_device_state_change(self):
        config = {'states': ['offline']}

        out = [
            'List of devices attached\nHTC\tdevice',
            'List of devices attached\nHTC\toffline'
        ]

        mock = Mock(side_effect=out)
        with patch.dict(adb.__salt__, {'cmd.run': mock}):

            ret = adb.beacon(config)
            self.assertEqual(ret, [])

            ret = adb.beacon(config)
            self.assertEqual(ret, [{'device': 'HTC', 'state': 'offline', 'tag': 'offline'}])

    def test_multiple_devices(self):
        config = {'states': ['offline', 'device']}

        out = [
            'List of devices attached\nHTC\tdevice',
            'List of devices attached\nHTC\toffline\nNexus\tdevice'
        ]

        mock = Mock(side_effect=out)
        with patch.dict(adb.__salt__, {'cmd.run': mock}):

            ret = adb.beacon(config)
            self.assertEqual(ret, [{'device': 'HTC', 'state': 'device', 'tag': 'device'}])

            ret = adb.beacon(config)
            self.assertEqual(ret, [
                {'device': 'HTC', 'state': 'offline', 'tag': 'offline'},
                {'device': 'Nexus', 'state': 'device', 'tag': 'device'}
            ])

    def test_no_devices_with_different_states(self):
        config = {'states': ['offline'], 'no_devices_event': True}

        mock = Mock(return_value='List of devices attached\nHTC\tdevice')
        with patch.dict(adb.__salt__, {'cmd.run': mock}):

            ret = adb.beacon(config)
            self.assertEqual(ret, [])

    def test_no_devices_no_repeat(self):
        config = {'states': ['offline', 'device'], 'no_devices_event': True}

        out = [
            'List of devices attached',
            'List of devices attached'
        ]

        mock = Mock(side_effect=out)
        with patch.dict(adb.__salt__, {'cmd.run': mock}):

            ret = adb.beacon(config)
            self.assertEqual(ret, [{'tag': 'no_devices'}])

            ret = adb.beacon(config)
            self.assertEqual(ret, [])

    def test_device_missing(self):
        config = {'states': ['device', 'missing']}

        out = [
            'List of devices attached\nHTC\tdevice',
            'List of devices attached',
            'List of devices attached\nHTC\tdevice',
            'List of devices attached\nHTC\tdevice'
        ]

        mock = Mock(side_effect=out)
        with patch.dict(adb.__salt__, {'cmd.run': mock}):

            ret = adb.beacon(config)
            self.assertEqual(ret, [{'device': 'HTC', 'state': 'device', 'tag': 'device'}])

            ret = adb.beacon(config)
            self.assertEqual(ret, [{'device': 'HTC', 'state': 'missing', 'tag': 'missing'}])

            ret = adb.beacon(config)
            self.assertEqual(ret, [{'device': 'HTC', 'state': 'device', 'tag': 'device'}])

            ret = adb.beacon(config)
            self.assertEqual(ret, [])

    def test_with_startup(self):
        config = {'states': ['device']}

        mock = Mock(return_value='* daemon started successfully *\nList of devices attached\nHTC\tdevice',)
        with patch.dict(adb.__salt__, {'cmd.run': mock}):
            ret = adb.beacon(config)
            self.assertEqual(ret, [{'device': 'HTC', 'state': 'device', 'tag': 'device'}])

    def test_with_user(self):
        config = {'states': ['device'], 'user': 'fred'}

        mock = Mock(return_value='* daemon started successfully *\nList of devices attached\nHTC\tdevice',)
        with patch.dict(adb.__salt__, {'cmd.run': mock}):
            ret = adb.beacon(config)
            mock.assert_called_once_with('adb devices', runas='fred')
            self.assertEqual(ret, [{'device': 'HTC', 'state': 'device', 'tag': 'device'}])

    def test_device_low_battery(self):
        config = {'states': ['device'], 'battery_low': 30}

        out = [
            'List of devices attached\nHTC\tdevice',
            '25',
        ]
        mock = Mock(side_effect=out)
        with patch.dict(adb.__salt__, {'cmd.run': mock}):
            ret = adb.beacon(config)
            self.assertEqual(ret, [{'device': 'HTC', 'state': 'device', 'tag': 'device'},
                                   {'device': 'HTC', 'battery_level': 25, 'tag': 'battery_low'}])

    def test_device_no_repeat(self):
        config = {'states': ['device'], 'battery_low': 30}

        out = [
            'List of devices attached\nHTC\tdevice',
            '25',
            'List of devices attached\nHTC\tdevice',
            '25'
        ]
        mock = Mock(side_effect=out)
        with patch.dict(adb.__salt__, {'cmd.run': mock}):
            ret = adb.beacon(config)
            self.assertEqual(ret, [{'device': 'HTC', 'state': 'device', 'tag': 'device'},
                                   {'device': 'HTC', 'battery_level': 25, 'tag': 'battery_low'}])

            ret = adb.beacon(config)
            self.assertEqual(ret, [])

    def test_device_no_repeat_capacity_increase(self):
        config = {'states': ['device'], 'battery_low': 75}

        out = [
            'List of devices attached\nHTC\tdevice',
            '25',
            'List of devices attached\nHTC\tdevice',
            '30'
        ]
        mock = Mock(side_effect=out)
        with patch.dict(adb.__salt__, {'cmd.run': mock}):
            ret = adb.beacon(config)
            self.assertEqual(ret, [{'device': 'HTC', 'state': 'device', 'tag': 'device'},
                                   {'device': 'HTC', 'battery_level': 25, 'tag': 'battery_low'}])

            ret = adb.beacon(config)
            self.assertEqual(ret, [])

    def test_device_no_repeat_with_not_found_state(self):
        config = {'states': ['offline'], 'battery_low': 30}

        out = [
            'List of devices attached\nHTC\tdevice',
            '25',
            'List of devices attached\nHTC\tdevice',
            '25'
        ]
        mock = Mock(side_effect=out)
        with patch.dict(adb.__salt__, {'cmd.run': mock}):
            ret = adb.beacon(config)
            self.assertEqual(ret, [{'device': 'HTC', 'battery_level': 25, 'tag': 'battery_low'}])

            ret = adb.beacon(config)
            self.assertEqual(ret, [])

    def test_device_battery_charged(self):
        config = {'states': ['device'], 'battery_low': 30}

        out = [
            'List of devices attached\nHTC\tdevice',
            '100',
        ]
        mock = Mock(side_effect=out)
        with patch.dict(adb.__salt__, {'cmd.run': mock}):
            ret = adb.beacon(config)
            self.assertEqual(ret, [{'device': 'HTC', 'state': 'device', 'tag': 'device'}])

    def test_device_low_battery_equal(self):
        config = {'states': ['device'], 'battery_low': 25}

        out = [
            'List of devices attached\nHTC\tdevice',
            '25',
        ]
        mock = Mock(side_effect=out)
        with patch.dict(adb.__salt__, {'cmd.run': mock}):
            ret = adb.beacon(config)
            self.assertEqual(ret, [{'device': 'HTC', 'state': 'device', 'tag': 'device'},
                                   {'device': 'HTC', 'battery_level': 25, 'tag': 'battery_low'}])

    def test_device_battery_not_found(self):
        config = {'states': ['device'], 'battery_low': 25}

        out = [
            'List of devices attached\nHTC\tdevice',
            '/system/bin/sh: cat: /sys/class/power_supply/*/capacity: No such file or directory',
        ]
        mock = Mock(side_effect=out)
        with patch.dict(adb.__salt__, {'cmd.run': mock}):
            ret = adb.beacon(config)
            self.assertEqual(ret, [{'device': 'HTC', 'state': 'device', 'tag': 'device'}])

    def test_device_repeat_multi(self):
        config = {'states': ['offline'], 'battery_low': 35}

        out = [
            'List of devices attached\nHTC\tdevice',
            '25',
            'List of devices attached\nHTC\tdevice',
            '40',
            'List of devices attached\nHTC\tdevice',
            '25',
            'List of devices attached\nHTC\tdevice',
            '80'
        ]
        mock = Mock(side_effect=out)
        with patch.dict(adb.__salt__, {'cmd.run': mock}):
            ret = adb.beacon(config)
            self.assertEqual(ret, [{'device': 'HTC', 'battery_level': 25, 'tag': 'battery_low'}])

            ret = adb.beacon(config)
            self.assertEqual(ret, [])

            ret = adb.beacon(config)
            self.assertEqual(ret, [{'device': 'HTC', 'battery_level': 25, 'tag': 'battery_low'}])

            ret = adb.beacon(config)
            self.assertEqual(ret, [])

    def test_weird_batteries(self):
        config = {'states': ['device'], 'battery_low': 25}

        out = [
            'List of devices attached\nHTC\tdevice',
            '-9000',
        ]
        mock = Mock(side_effect=out)
        with patch.dict(adb.__salt__, {'cmd.run': mock}):
            ret = adb.beacon(config)
            self.assertEqual(ret, [{'device': 'HTC', 'state': 'device', 'tag': 'device'}])

    def test_multiple_batteries(self):
        config = {'states': ['device'], 'battery_low': 30}

        out = [
            'List of devices attached\nHTC\tdevice',
            '25\n40',
        ]
        mock = Mock(side_effect=out)
        with patch.dict(adb.__salt__, {'cmd.run': mock}):
            ret = adb.beacon(config)
            self.assertEqual(ret, [{'device': 'HTC', 'state': 'device', 'tag': 'device'},
                                   {'device': 'HTC', 'battery_level': 25, 'tag': 'battery_low'}])

    def test_multiple_low_batteries(self):
        config = {'states': ['device'], 'battery_low': 30}

        out = [
            'List of devices attached\nHTC\tdevice',
            '25\n14',
        ]
        mock = Mock(side_effect=out)
        with patch.dict(adb.__salt__, {'cmd.run': mock}):
            ret = adb.beacon(config)
            self.assertEqual(ret, [{'device': 'HTC', 'state': 'device', 'tag': 'device'},
                                   {'device': 'HTC', 'battery_level': 25, 'tag': 'battery_low'}])
